/*******************************************************************************

NAME            sdram.c
VERSION         %version: 18 %
UPDATE DATE     %date_modified: Tue Nov  5 11:46:54 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright(C) 2012 LSI Corporation.                                     *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  sdram.c
 *          Title:  File containing functions for initialization of DDR
 *     Programmer:  Naresh Madhusudana
 *  Creation Date:  Aug 15, 2011
 *
 *
 *  Version History 
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  15/08/11  NM   This file has the DDR3 SDRAM initialization function calls
 *  03/27/12  JM   Porting changes for Dual core environment in LSISAS3108.
 *
 *
 * Description
 * ------------
 *
 *    This file consists of function calling the entry point of standalone
 *    library to initialize DDR3 memory.
 *
 *-------------------------------------------------------------------------
 */

#include <common.h>
#include <asm/mmu.h>
#include <i2c.h>
#include "include/lsisas3108_common.h"
#include "boardDisplayCode.h"


#define DDR3_TRAINING_TLB_MEM_REGION_SIZE   (4*1024)

#define IS_POWERUP()    ( (*(u8*)(CONFIG_SYS_FPGA_BASE + 0x24)) & 0x01 ) 

extern unsigned long do_go_exec (ulong (*entry)(int, char * const []), int argc, char * const argv[]);
extern void iopMemSetZero(u32, u32);

void init_mem_region(u64 phyAddr, u32 size);
int callMiniDMA(u64 ddr3Addr , u32 ddr3Size);
int spdCheck(u8* spdData);

/********************************************************************
 *
 *   M E M O R Y C O N T R O L L E R - I N I T I A L I Z A T I O N
 *
 ********************************************************************/

/*
 * Name: initdram(board_type)
 *
 * Description: This fucntion is used to initialize the DDR3 memory
 *
 *
 * Arguments : board type: any specific board information
 *
 * Return value : None
 */

phys_size_t initdram(int board_type)
{
    u8    DDR3_SPD[256] = {0};
    char  *entry_point  = stringify(LSISAS3108_STANDALONE_ENTRY);
    int   lib_name      = 1;
    int   func_name     = 1;
    char  *caddr[5];
    u32   addr;
    u64   rc;
    u64   phys_addr;
    u32   virt_addr;
    u32   probeCnt;

    u64 startddrPhyAddrAlias = 0x400000000ULL;
    u64 ddrPhyAddrAlias;
    U8  uninitailizedDDRSizeinGB = 0;

#ifdef CONFIG_MENUPROMPT
    /* Display menu prompt here instead of in main.c so that MFG scripts
     * have as much time as possible to respond. Also, printing before
     * resets are enabled below guarantees that we'll see the prompt. */
    serial_puts(CONFIG_MENUPROMPT);
#endif

    /* Remove the default 4K page entry at the top of flash. change_tlb() doesn't
     * compare the size properly and will end up remapping this instead of the
     * 0xff000000 entry below if we don't remove it.
     * We should probably fix change_tlb() sometime. */
    remove_tlb(0xfffff000, 0x1000);

    /* Map first 256MB of DDR3 - considering it is the minimum DIMM size */
    /* As per the DDR3 lib recommendation to avoid the speculative fetch , we need the cached DDR3 calibration memory
     * as L1 and L2 cache inhibited , L3 cache enabled */
    program_tlb(DDR3_PHY_ADDR_ALIAS_BASE, DDR3_VIRT_ADDR_1ST_RANK_CACHED,        DDR3_TRAINING_TLB_MEM_REGION_SIZE, TLB_WORD2_I_ENABLE);
    program_tlb(DDR3_PHY_ADDR_ALIAS_BASE_NON_CACHED, DDR3_VIRT_ADDR_1ST_RANK_NON_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE, TLB_WORD2_I_ENABLE);

    /* 1GB */
    program_tlb(DDR3_PHY_ADDR_ALIAS_BASE_1GB, DDR3_VIRT_ADDR_1GB_2ND_RANK_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE, TLB_WORD2_I_ENABLE);
    program_tlb(DDR3_PHY_ADDR_ALIAS_BASE_1GB_NON_CACHED, DDR3_VIRT_ADDR_1GB_2ND_RANK_NON_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE, TLB_WORD2_I_ENABLE);

    /* 2GB */
    program_tlb(DDR3_PHY_ADDR_ALIAS_BASE_2GB,DDR3_VIRT_ADDR_2GB_2ND_RANK_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE, TLB_WORD2_I_ENABLE);
    program_tlb(DDR3_PHY_ADDR_ALIAS_BASE_2GB_NON_CACHED,DDR3_VIRT_ADDR_2GB_2ND_RANK_NON_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE, TLB_WORD2_I_ENABLE);
    
    /* 4GB */
    program_tlb(DDR3_PHY_ADDR_ALIAS_BASE_4GB,DDR3_VIRT_ADDR_4GB_2ND_RANK_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE, TLB_WORD2_I_ENABLE);
    program_tlb(DDR3_PHY_ADDR_ALIAS_BASE_4GB_NON_CACHED,DDR3_VIRT_ADDR_4GB_2ND_RANK_NON_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE, TLB_WORD2_I_ENABLE);

    /* TLB entry for RACC */
    program_tlb(0x10C20D0000ULL, 0x90000000, TLB_4KB_SIZE ,(TLB_WORD2_I_ENABLE | TLB_WORD2_E_ENABLE));

    phys_addr = CONFIG_SYS_SDRAM_BASE;
    virt_addr = CONFIG_SYS_SDRAM_BASE;

    phys_addr += DDR_SIZE_256MB;
    virt_addr += DDR_SIZE_256MB;

    boardDisplayCode(UBOOT_DDR3_SPD_READ_START);
    for (probeCnt=0; probeCnt<10; probeCnt++)
    {
        if (i2c_probe(CONFIG_SYS_I2C_SLAVE) == 0)
        {
            printf("\ni2c probe count: %d\n", probeCnt);
            if (i2c_read(CONFIG_SYS_I2C_SLAVE, 0, 1, DDR3_SPD, 256) == 0)
            {
                printf("SPD Reading. Success!\n");
                /* FIXME: Remove this once i2c issues are resolved, but it is
                   useful for debugging if errors do occur. */
                print_buffer((ulong)DDR3_SPD, DDR3_SPD, 1, 256, 0);
                break;
            }
            else
            {
                printf("SPD Reading. Failed!\n");
                boardDisplayCode (UBOOT_DDR3_SPD_READ_FAILURE);
                /* FIXME: Remove this once i2c issues are resolved, but it is
                   useful for debugging if errors do occur. */
                print_buffer((ulong)DDR3_SPD, DDR3_SPD, 1, 256, 0);
                hang();
            }
        }
    }

    if (probeCnt == 10)
    {
        boardDisplayCode(UBOOT_DDR3_NO_DIMM_FOUND);
        printf("DIMM not found!\n");
        hang();
    }

    /* Reset Elektra manually */
    mtdcr(0xE00C,0x003C0000);
    udelay(10);
    mtdcr(0xE00D,0x03800000);
    udelay(10);
    mtdcr(0xE00C,0x00000000);
    mtdcr(0xE00D,0x00000000);
    udelay(10);

    /* Sanity check the DIMM before starting training */
    if (spdCheck(DDR3_SPD))
    {
        boardDisplayCode(UBOOT_UNSUPPORTED_DIMMS);
        printf("*** Unsupported DIMM ***\n");
        hang();
    }

    boardDisplayCode(UBOOT_DDR3_TRAINING_START);
    caddr[0] = entry_point;
    caddr[1] = (char *)(&lib_name);
    caddr[2] = (char *)(&func_name);
    caddr[3] = (char *) DDR3_SPD;
    addr     = (LSISAS3108_STANDALONE_ENTRY);
    rc       = do_go_exec ((void *)addr, 4, caddr);

    if( !rc )
    {
        /* Init failed */
        printf("DDR3 INIT FAILED! 0x%llx\r\n", rc);
        hang();
    }
    else
    {
        printf("DDR3 init completed successfully 0x%llx\n", rc);
    }

    boardDisplayCode(UBOOT_DDR3_TRAINING_COMPLETE);

    remove_tlb(DDR3_VIRT_ADDR_1ST_RANK_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE);
    remove_tlb(DDR3_VIRT_ADDR_1ST_RANK_NON_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE);
    remove_tlb(DDR3_VIRT_ADDR_1GB_2ND_RANK_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE);
    remove_tlb(DDR3_VIRT_ADDR_1GB_2ND_RANK_NON_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE);
    remove_tlb(DDR3_VIRT_ADDR_2GB_2ND_RANK_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE);
    remove_tlb(DDR3_VIRT_ADDR_2GB_2ND_RANK_NON_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE);
    remove_tlb(DDR3_VIRT_ADDR_4GB_2ND_RANK_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE);
    remove_tlb(DDR3_VIRT_ADDR_4GB_2ND_RANK_NON_CACHED, DDR3_TRAINING_TLB_MEM_REGION_SIZE);

   /* We have completed the DDR3 training successfully , now we need to
    * initailize the whole DDR with zeros
    */

    /* Now we need a CRAM TLB entry LE+uncached;Remove the existing and create a new entry */
    remove_tlb(CONFIG_SYS_ISRAM_BASE3, 1*1024*1024);
    program_tlb(CONFIG_SYS_ISRAM_BASE3, CONFIG_SYS_ISRAM_BASE3, 1*1024*1024, (TLB_WORD2_I_ENABLE|TLB_WORD2_E_ENABLE));

    printf("Initializing DDR (# = 1GB ) : ");
    
    uninitailizedDDRSizeinGB = rc/1024;

    ddrPhyAddrAlias = startddrPhyAddrAlias;

    if ( IS_POWERUP() )
    {
        while (uninitailizedDDRSizeinGB)
        {
            init_mem_region(ddrPhyAddrAlias, DDR_SIZE_1GB);
            ddrPhyAddrAlias +=DDR_SIZE_1GB;
            uninitailizedDDRSizeinGB --;
            printf("#");
        }
    }
    else 
    {
        /* Initalize the first 256 MB of memory only */

        printf("Initializing First 256MB memory...");
        init_mem_region(ddrPhyAddrAlias, DDR_SIZE_256MB);
    }

    printf(" Finished !!  \n ");

    /* DDR TLBs are removed by above loop, set them up here for uboot driver
     * and OS loading use. */
    program_tlb(CONFIG_PHY_OS_BASE,        CONFIG_OS_BASE,        DDR_SIZE_256MB, 0);
    program_tlb(CONFIG_PHY_OS_BUF_BASE1,   CONFIG_OS_BUF_BASE1,   DDR_SIZE_16MB,  0);
    program_tlb(CONFIG_PHY_OS_BUF_BASE2,   CONFIG_OS_BUF_BASE2,   DDR_SIZE_16MB,  0);
    program_tlb(CONFIG_PHY_BE_DRIVER_BASE, CONFIG_BE_DRIVER_BASE, DDR_SIZE_16MB,  TLB_WORD2_I_ENABLE);
    program_tlb(CONFIG_PHY_LE_DRIVER_BASE, CONFIG_LE_DRIVER_BASE, DDR_SIZE_16MB,  TLB_WORD2_I_ENABLE | TLB_WORD2_E_ENABLE);

    /*
     * Since we are still operating out of flash, and because of the issue
     * noted below that forces a 2G DIMM size to be returned, we explicitly 
     * put the DIMM size in MB into OCM.  As we are later booting the OS we
     * will take this and format it as needed into the appropriate boot string.
     */
    *((u32 *)CONFIG_SYS_ISRAM_BASE0) = (u32)rc;

    /* Converting DDR3 RAM size from MB to GB */
    rc *= 1024*1024;

    /* Forcing size of 2GB if greater as this is causing a crash */
    if (rc > (u64)2*1024*1024*1024)
    {
        rc = (u64)2*1024*1024*1024;
    }

    asm volatile ("msync");

    /* Restore the BE+cached TLB entry, we will execute code out of OCM later */
    remove_tlb(CONFIG_SYS_ISRAM_BASE3, 1*1024*1024);
    program_tlb(CONFIG_SYS_ISRAM_BASE3, CONFIG_SYS_ISRAM_BASE3, 1*1024*1024, 0);

#if defined(CONFIG_476I) && defined(CONFIG_DUALCORE)
    return (((phys_size_t)rc - DDR_SIZE_512MB));
#else
    return ((phys_size_t)rc);
#endif
}

/*
 * Name: board_add_ram_info(void)
 *
 * Description: This fucntion is used to print any board specific information
 *
 * Arguments : None
 *
 * Return value : None
 */
void board_add_ram_info(u32 use_default)
{
    printf(" (ECC is PRESENT)");
}

/*
 * Name: init_mem_region(u64 phyAddr, u32 size)
 *
 * Description: Maps a 1GB TLB containing the memory region and inits it
 *   to zero. Can be optionally compiled to use either the CPU or DMA
 *   operations to perform the init.
 *
 * Arguments :
 *    phyAddr - the start address of the DDR3 to be initialized
 *    size - size of the DDR3 to be initialized (can't cross 1 GB boundary)
 * Return value :
 *    nothing
 */
void init_mem_region(u64 phyAddr, u32 size)
{
    /* Align base to 1GB */
    u64 phyBase = phyAddr & 0xFC0000000ULL;

    /* Map 1GB TLB at physical addr to virt addr 0 (cache enabled) */
    program_tlb(phyBase, 0, DDR_SIZE_1GB, 0);

#if 0
    /* Get offset inside 1GB region */
    u32 startOffset = (u32)phyAddr & 0x3FFFFFFF;

    /* Use CPU to fill desired virtual addresses with 0's */
    iopMemSetZero(startOffset, size);
    asm volatile ("msync");
#else
    /* Call miniDMA to perform the initialization */
    callMiniDMA(phyAddr, size);
#endif
    /* Remove 1GB TLB */
    remove_tlb(0, DDR_SIZE_1GB);
}

/*
 * Name: callMiniDMA(u64 ddr3Addr , u64 ddr3Size)
 *
 * Description: This fucntion calls the miniDMA routine to initialize the DDR
 *
 * Arguments :
 *    ddr3Addr - the start address of the DDR3 to be initialized
 *    ddr3Size - size of the DDR3 to be initialized.
 * Return value :
 *    int - mini dma status
 */
int callMiniDMA(u64 ddr3Addr , u32 ddr3Size)
{
    int miniDMAstatus;
    U32 addr;
    char *entry_point = stringify(LSISAS3108_STANDALONE_ENTRY);
    int  lib_name               = 2;
    int  func_name              = 1;
    char *caddr[8];

    caddr[0] = entry_point;
    caddr[1] = (char *)(&lib_name);
    caddr[2] = (char *)(&func_name);
    caddr[3] = (char *)(&ddr3Addr);
    caddr[4] = (char *)(&ddr3Size);
    addr     = (LSISAS3108_STANDALONE_ENTRY);
    miniDMAstatus = do_go_exec ((void *)addr, 5, caddr);
    return miniDMAstatus;

}


/* 
 * Name: spdCheck(u8* spdData)
 *
 * Description: This function checks to see if a DIMM appears to be valid
 *
 * Arguments :
 *    spdData - pointer to SPD data read from the DIMM
 * Return value :
 *    int - 0 for valid DIMM, 1 for invalid DIMM
 *
 * Notes:
 * To calculate the total capacity of a DIMM, the JEDEC SPD standard says to
 * use the following formula:
 *   SDRAM CAPACITY / 8 * PRIMARY BUS WIDTH / SDRAM WIDTH * RANKS
 * Where: 
 *   SDRAM CAPACITY = SPD byte 4 bits 3:0
 *   PRIMARY BUS WIDTH = SPD byte 8 bits 2:0
 *   SDRAM WIDTH = SPD byte 7 bits 2:0
 *   RANKS = SPD byte 7 bits 5:3
 * In our case, PRIMARY BUS WIDTH is always 64, so we can shorten this to:
 *   SDRAM CAPACITY * 8 / SDRAM WIDTH * RANKS
 * Rearranged, it looks like this:
 *   SDRAM CAPACITY * 8 * RANKS / SDRAM WIDTH
 * Dividing by SDRAM WIDTH can easily be turned into a right shift
 * operation since the bits 2:0 represent sizes in powers of two:
 * 000=4, 001=8, 010=16, 011=32 - so the total right shift value is
 * 2 + (bits 2:0). So now the equation is: 
 *   (SDRAM CAPACITY * 8 * RANKS) >> SDRAM WIDTH SHIFT
 * SDRAM CAPACITY is in Mbits and the end result will be MBytes */

int spdCheck(u8* spdData)
{
    u8 invalid_dimm = 0;

    u8 dram_type   = spdData[2];  // Must be 0xB (DDR3)
    u8 module_type = spdData[3];  // Must be 0x1 (RDIMM)
    u8 bus_width   = spdData[8];  // Must be 0xB (64 data bits + 8 ECC bits)
    u8 tCK_min     = spdData[12]; // Can't be larger than 0x0A

    u32 capacity_Mb = 256 << (spdData[4] & 0x0F);  // capacity in Mbits
    u8 ranks = ((spdData[7] >> 3) & 0x07) + 1;     // check this, must be 1 or 2
    u8 sdram_shift = 2 + (spdData[7] & 0x7);       // number of bits to >> by in calculation below

    /* Get DIMM size in MB - see notes in function header above */
    u32 size_MB = (capacity_Mb * 8 * (u32)ranks) >> sdram_shift;

    /* Verify that it is a DDR3 DIMM */
    if (dram_type != 0x0B)
    {
        printf("Invalid dram_type: 0x%02x\n", dram_type);
        invalid_dimm = 1;
    }

    /* If somebody plugs in the wrong type of DDR3 DIMM */
    if (module_type != 0x01)
    {
        printf("Invalid module_type: 0x%02x\n", module_type);
        invalid_dimm = 1;
    }

    /* This will detect if someone plugs in a non-ECC DIMM */
    if (bus_width != 0x0B)
    {
        printf("Invalid bus_width: 0x%02x\n", bus_width);
        invalid_dimm = 1;
    }

    /* Only single and double rank DIMMs allowed (No quad or more) */
    if (ranks > 2)
    {
        printf("Invalid number of ranks: 0x%02x\n", ranks);
        invalid_dimm = 1;
    }

    /* If tCK_min is > 0x0A then the DIMM is slower than 1600MHz, which is the
     * slowest speed allowed on Tahoe. */
    if (tCK_min > 0x0A)
    {
        printf("Unsupported DIMM speed: 0x%02x\n", tCK_min);
        invalid_dimm = 1;
    }

    /* Only 2GB, 4GB and 8GB DIMMs are allowed */
    if ((size_MB != 2048) && (size_MB != 4096) && (size_MB != 8192))
    {
        printf("Unsupported DIMM size: 0x%04d MB\n", size_MB);
        invalid_dimm = 1;
    }

    /* We can't persist memory across reboots on a 2GB dual-rank DIMM */
    if ((size_MB == 2048) && (ranks == 2))
    {
        printf("Unsupported dual-rank 2GB DIMM\n");
        invalid_dimm = 1;
    }

    return invalid_dimm;
}

