/***************************************************************************
 *                                                                         *
 *  Copyright(c) 2003-2011 LSI Corporation.  All rights reserved.          *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiConfig.h
 *          Title:  SAS Firmware IOP Interface Config Structures and Definitions
 *     Programmer:  Steve Johnson
 *  Creation Date:  June 11, 2003
 *
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface config structures and
 * definitions.
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPICONFIG_H_INCLUDED
#define IOPICONFIG_H_INCLUDED

/******************************************************************************
*
*      I O P I   C O N F I G   S T R U C T S   A N D   D E F I N I T I O N S
*
******************************************************************************/

/* Steve Schremmer said to put product specific page versions in this file. */
#define MPI2_EXTENDED_MANUFACTURING40_PAGEVERSION     (0x00)
#define MPI2_EXTENDED_MANUFACTURING41_PAGEVERSION     (0x00)


/*****************************************************************************
*
*  Manufacturing Page 6 Config Page
*
*****************************************************************************/

/*
 * The number of entries for this page needs to be the MAXIMUM number of GPIO's supported by the build.
 *
 *  Meteor = 56
 *  Falcon = 36
 *  Liberator = 32
 *
 * Therefore current support for 56.

 * This value needs to be synchronized with the value used to
 * override MPI2_IO_UNIT_PAGE_3_GPIO_VAL_MAX.
 */
#ifndef IOC_MFG_PAGE6_GPIO_DEFS
#define IOC_MFG_PAGE6_GPIO_DEFS                     (MPI2_IO_UNIT_PAGE_3_GPIO_VAL_MAX)
#endif

/* GPIODefinition */
typedef struct _IOC_CFG_MFG_6_GPIO_DEF
{
    U8                                  FunctionCode;                                      /* 00h */
    U8                                  Flags;                                             /* 01h */
    U8                                  Param1;                                            /* 02h */
    U8                                  Param2;                                            /* 03h */
    U32                                 Param3;                                            /* 04h */
} IOC_CFG_MFG_6_GPIO_DEF, *PTR_IOC_CFG_MFG_6_GPIO_DEF;

/* All GPIO Definition Flags definitions */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_MASK                     (0x03)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_INPUT                    (0x00)    /* Pad may be used as an input */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_OUTPUT_OPEN_DRAIN        (0x01)    /* 1: High impedance, 0: Enabled, pull up required*/
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_OUTPUT_OPEN_SOURCE       (0x02)    /* 1: Enabled 0: High Impedance, pull down required */
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_DIR_OUTPUT_PUSH_PULL         (0x03)    /* Always driving */

#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ACTIVE_MASK                  (0x04)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ACTIVE_LOW                   (0x00)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ACTIVE_HIGH                  (IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ACTIVE_MASK)

#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_MASK                  (0x30)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_GPIO                  (0x00)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_SAS_LED               (0x10)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_RSVD                  (0x20)
#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_TRNG                  (IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_MASK)

#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_ALTSEL_SH                    (4)

#define IOC_CFG_MFG_6_GPIO_DEF_FLAGS_BIDIR_ENABLE                 (0x08)

/* GPIO Function code definitions (8 bits) */
#define IOC_CFG_MFG_6_GPIO_FUNCTION_UNKNOWN                       (0x0)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_DEFAULT                       (0x1)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_IR_DRIVE_FAULT                (0x2)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_DA_SEP_RESET                  (0x3)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_DA_SEP_DETECT                 (0x4)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_SEP_DETECT_AND_RESET          (0x5)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_UNUSED_6                      (0x6)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_UNUSED_7                      (0x7)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_BKPLN_MGMT_TYPE               (0x8)

#define IOC_CFG_MFG_6_GPIO_FUNCTION_DRIVE_ACTIVITY_LED            (0x9)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_PORT_STATUS_GREEN_LED         (0xA)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_PORT_STATUS_YELLOW_LED        (0xB)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_HEARTBEAT_LED                 (0xC)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_DA_SEP_CONNECT                (0xD)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_I2C_MUX_RESET                 (0xE)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_MFG3_MUX_BIT                  (0xF)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_INTERRUPT                     (0x10)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_FAULT_INDICATION              (0x11)
#define IOC_CFG_MFG_6_GPIO_FUNCTION_IBUTTON                       (0x12)
                                                                            /* FC 0x14-0x16 are used by SSS HBA */
#define IOC_CFG_MFG_6_GPIO_FUNCTION_SAS_HD_CABLE_MGMT             (0x17)    /* Mini SAS HD Cable Mgmt */

#define IOC_CFG_MFG_6_GPIO_FUNCTION_VENDOR_SPECIFIC               (0xC0)    /* 0xC0 - 0xFF are Vendor Specific */


/* Param1 definitions for INTERRUPT Function */
#define IOC_CFG_MFG_6_INT_PARAM1_TYPE_MASK                        (0x01)
#define IOC_CFG_MFG_6_INT_PARAM1_EDGE_SENSITIVE                   (0x00)
#define IOC_CFG_MFG_6_INT_PARAM1_LEVEL_SENSITIVE                  (0x01)

#define IOC_CFG_MFG_6_INT_PARAM1_SOURCE_MASK                      (0xF0)
#define IOC_CFG_MFG_6_INT_PARAM1_SOURCE_GENERIC                   (0x00)  /* For backward compatible */
#define IOC_CFG_MFG_6_INT_PARAM1_SOURCE_HD_CABLE_MGMT             (0x10)  /* HD Cable Management interrupt */
#define IOC_CFG_MFG_6_INT_PARAM1_SOURCE_ACTIVE_CABLE_OVERCURRENT  (0x20)  /* HD Active Cable over current interrupt */

/* Param1 definitions for SAS HD CABLE MGMT Function */
#define IOC_CFG_MFG_6_HD_CABLE_MGMT_PARAM1_MOD_PRESENT            (0x00)  /* SAS HD cable module present signal */
#define IOC_CFG_MFG_6_HD_CABLE_MGMT_PARAM1_ACTIVE_PWR_ENABLE      (0x01)  /* Active Cable power enable signal */
#define IOC_CFG_MFG_6_HD_CABLE_MGMT_PARAM1_MGMT_PWR_ENABLE        (0x02)  /* Cable Mgmt interface power enable signal */

/* Virtual GPIO Definitions */
#define IOC_CFG_MFG_6_GPIO_VIRT_ALTSIO_RES0                       (0x28)    /* Virtual GPIO for AltSIOFunc Res0 (bit 8)*/
#define IOC_CFG_MFG_6_GPIO_VIRT_ALTSIO_RES1                       (0x29)    /* Virtual GPIO for AltSIOFunc Res1 (bit 9)*/


/* GPIO Defines */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_6
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                            /* 00h */
    U8                                  NumGPIO;                                           /* 04h */
    U8                                  Reserved1;                                         /* 05h */
    U16                                 Flags;                                             /* 06h */
    U32                                 Reserved2;                                         /* 08h */
    U32                                 Reserved3;                                         /* 0Ch */
    IOC_CFG_MFG_6_GPIO_DEF              GPIODefinition[IOC_MFG_PAGE6_GPIO_DEFS];           /* 10h */
} IOC_CONFIG_PAGE_MFG_PAGE_6, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_6;

#define IOC_CONFIG_PAGE_MFG6_FLAGS_HBLED_DISABLE                  (0x0001)  /* 1: Disable Heartbeat 0: Enable Heartbeat */
#define IOC_CONFIG_PAGE_MFG6_FLAGS_RESERVED_1_31                  (0xFFFE)


/*****************************************************************************
*
*  Manufacturing Page 8 Config Page
*
*****************************************************************************/

/*
 * Leave this define set to one and check Header.PageLength at runtime.
 */

#ifndef IOC_MAN_PAGE8_ISTWI_INFO_ENTRIES
#define IOC_MAN_PAGE8_ISTWI_INFO_ENTRIES         (1)
#endif

/* Istwi Controller information */
typedef struct _ISTWI_CONTROLLER_INFO
{
    U16                         SlaveAddr;                                         /* 00h */
    U16                         Flags;                                             /* 04h */
} ISTWI_CONTROLLER_INFO, *PTR_ISTWI_CONTROLLER_INFO;

/* ISTWI controller flags fields */
#define IOC_CFG_MFG_8_ISTWI_CTRL_FLAGS_ADDR_SIZE_10_BIT       (0x0001)
#define IOC_CFG_MFG_8_ISTWI_CTRL_FLAGS_ADDR_SIZE_07_BIT       (0x0000)

#define IOC_CFG_MFG_8_ISTWI_FLAGS_SLAVE_ADDR_SIZE_MASK        (0x0001)

#define IOC_CFG_MFG_8_ISTWI_FLAGS_DISABLE                     (0x0080)            /* Disable this controller when set */
#define IOC_CFG_MFG_8_ISTWI_FLAGS_ISTWI_SIO_ROUTING           (0x02)              /* ISTWI route to sideband when set */

/* Istwi Device information */
typedef struct _ISTWI_DEVICE_INFO
{
    U32                         Flags;                                             /* 00h */
    U8                          DeviceType;                                        /* 04h */
    U8                          BusNum;                                            /* 05h */
    U16                         DeviceAddr;                                        /* 06h */
    U8                          MuxType;                                           /* 08h */
    U8                          MuxChannel;                                        /* 09h */
    U16                         MuxAddr;                                           /* 0Ah */
    U32                         DeviceSpecific[2];                                 /* 0Ch */
} ISTWI_DEVICE_INFO, *PTR_ISTWI_DEVICE_INFO;

/* ISTWI Device Flags field defines */
#define IOC_CFG_MFG_8_ISTWI_FLAGS_DEV_DATA_OFFSET_MASK          (0x00000700)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_DEV_BUS_SPEED_MASK            (0x000000F0)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_DEV_BUS_SPEED_100_KHZ         (0x00000000)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_DEV_BUS_SPEED_400_KHZ         (0x00000010)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_DEV_ADDR_SIZE_MASK            (0x00000004)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_MUX_ADDR_SIZE_MASK            (0x00000002)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_ADDR_SIZE_10_BIT              (0x00000001)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_ADDR_SIZE_07_BIT              (0x00000000)

#define IOC_CFG_MFG_8_ISTWI_FLAGS_MUX_PRESENT_MASK              (0x00000001)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_MUX_PRESENT                   (IOC_CFG_MFG_8_ISTWI_FLAGS_MUX_PRESENT_MASK)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_MUX_NOT_PRESENT               (0x00000000)

#define IOC_CFG_MFG_8_ISTWI_FLAGS_DEV_DATA_OFFSET_SHIFT         (8)
#define IOC_CFG_MFG_8_ISTWI_FLAGS_DEV_BUS_SPEED_SHIFT           (4)

/* ISTWI device types */
#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_UNKNOWN                    (0x00)
#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_BOOT_ROM                   (0x01)  /* SBR */
#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_SEP                        (0x02)
#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_DDR_SPD                    (0x03)
#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_MUX                        (0x04)
#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_SIGNAL_CONDITIONER         (0x05)
#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_SEEPROM                    (0x06)
#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_TEMP_SENSOR                (0x07)

#define IOC_CFG_MFG_8_ISTWI_DEV_TYPE_VENDOR_SPECIFIC            (0xE0)  /* 0xE0 - 0xFF are Vendor Specific */

/*
 * An ISTWI "controller" is correlated with externally available busses
 * These may be muxed internally to the chip, however, this detail
 * should be hidden from the user except for delays that may occur
 * due to resource conflicts
 */
#ifndef IOC_NUM_ISTWI_CONTROLLERS_MAX
#define IOC_NUM_ISTWI_CONTROLLERS_MAX   (6)
#endif

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_8
{
    MPI2_CONFIG_PAGE_HEADER     Header;                                            /* 00h */
    ISTWI_CONTROLLER_INFO       ControllerInfo[IOC_NUM_ISTWI_CONTROLLERS_MAX-1];   /* 04h */
    U8                          NumIstwiDevices;                                   /* 18h */
    U8                          ControllerInfo5_Flags;                             /* 19h */
    U16                         ControllerInfo5_SlaveAddr;                         /* 1Ah */
    ISTWI_DEVICE_INFO           IstwiDevInfo[IOC_MAN_PAGE8_ISTWI_INFO_ENTRIES];    /* 1Ch */
} IOC_CONFIG_PAGE_MFG_PAGE_8, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_8;

typedef struct
{
    U32 Maximum;        /* Maximum Number Of Resources allowed */
    U32 Decrement;      /* Amount Subtracted until Minimum is reached */
    U32 Minimum;        /* Minimum Number Of Resources allowed */
    U32 Actual;         /* Actual Number of Resources Allocated if Successful (May be zero in NVStore) */
} IOC_CONFIG_MFG9_RESOURCE;

typedef enum
{
    IOC_CFG_MFG9_RESOURCE_NUM_VFS                       = 0,    /* Number of Virtual Functions Supported (may be less than MAX) */
    IOC_CFG_MFG9_RESOURCE_NUM_VPS                       = 1,    /* Number of Virtual Ports Supported */
    IOC_CFG_MFG9_RESOURCE_HOST_CREDITS_PER_VF           = 2,    /* Number of Global Credits per VF */
    IOC_CFG_MFG9_RESOURCE_HI_PRIORITY_Q_DEPTH_PER_VF    = 3,    /* Number of HiPQ Credits per VF */
    IOC_CFG_MFG9_RESOURCE_TARGETS                       = 4,    /* Number of Targets supported */
    IOC_CFG_MFG9_RESOURCE_INITIATORS                    = 5,    /* Number of Initiators supported */
    IOC_CFG_MFG9_RESOURCE_TGT_CMD_BUFFERS_PER_VP        = 6,    /* Number of Target Cmd Buffers per VP*/
    IOC_CFG_MFG9_RESOURCE_EXPANDERS                     = 7,    /* Number of Expanders Supported */
    IOC_CFG_MFG9_RESOURCE_PHYS                          = 8,    /* Max number of phys supported per enclosure */
    IOC_CFG_MFG9_RESOURCE_ENCLOSURES                    = 9,    /* Max Enclosures supported */
    IOC_CFG_MFG9_RESOURCE_RING_BUF_SIZE                 =10,    /* Size of ring buffer in bytes */
    IOC_CFG_MFG9_RESOURCE_IR_BUFFER_SIZE                =11,    /* Size of RAID resync buffer in bytes */
    IOC_CFG_MFG9_RESOURCE_NUM_ROUTE_TABLE_ENTRIES       =12,    /* Number of expander route table entries */
#if COMPILE_OPTION_SAS_PORT_VIRT
    IOC_CFG_MFG9_RESOURCE_NUM_VPHYS_PER_VE              =13,    /* Number of VPhys per VE */
    IOC_CFG_MFG9_RESOURCE_NUM_VES                       =14,    /* Number of VE*/
#endif  /* COMPILE_OPTION_SAS_PORT_VIRT  */
    IOC_CFG_MFG9_RESOURCE_CHAIN_ELEMENTS_PER_VF         =15,    /* Number of Local Chain Buffer Elements per VF */
    /* This must always be the last member in this ENUM! */
    IOC_CFG_MFG9_NUMBER_OF_RESOURCES            /* Number of resources supported by this build */
} IOC_CONFIG_MFG9_RESOURCE_ENUM;

/*
 * Other RTConfig values and their sources:
 * HostFrameSize - Size of host frame from IOCInit
 * IOCRequestFrameSize - CONSTANT, Build Option
 * PLIocRequestFrameSize - Variable only in PL builds (PL should not use the CONSTANT
 * NumResourceFrames - 128 for all builds
 * MaxVirtualFunctions - from hardware
 */

/* misc stuffs moved from mfg page 3 */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_9
{
    MPI2_CONFIG_PAGE_HEADER     Header;                                            /* 00h */
    U32                         MaxAttempts;    /* Maximum Interations before FAULT */
    U32                         NumResources;   /* Number of resource described in Resource Array */
    U16                         MessageSize;    /* Size of message frame in bytes (1024 max; integral multiple of 16) */
    U16                         Flags;          /* Misc. configuration values */
    U16                         ChainSize;      /* Size of a Local Chain Buffer Element in 16 byte lines (0 interpreted as 1MB) *//* 10h */
    U16                         Reserved12;                                        /* 12h */
    IOC_CONFIG_MFG9_RESOURCE    ResourceArray[IOC_CFG_MFG9_NUMBER_OF_RESOURCES];   /* 14h */
} IOC_CONFIG_PAGE_MFG_PAGE_9, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_9;

#define IOC_CFG_MFG_PG9_FLAG_NUM_ADDL_SGLS_MASK    (0x03)
#define IOC_CFG_MFG_PG9_FLAG_ENABLE_BIDIR_CONTEXT  (0x04)

typedef struct _IOC_CONFIG_UART_SETTINGS
{
    U8      Flags;
    U8      Reserved_MoreFlags;
    U16     Reserved_TO;
    U32     BaudRate;
} IOC_CONFIG_UART_SETTINGS;

#define IOC_CONFIG_UART_FLAGS_USE_EXT_CLK       (0x80) /* 0x80: Use ext clock, 0x00: Use internal clock (30Mhz) */
#define IOC_CONFIG_UART_FLAGS_USE_XON_XOFF      (0x40) /* 0x40: Use XON/XOFF, 0x00: No Flow Control */
#define IOC_CONFIG_UART_FLAGS_DISABLED          (0x20) /* 0x20: 0: Init and use this UART, 1: Disable UART usage */

typedef struct _IOC_CPLTQ_INT_CONFIG
{
    U32     SasCoreCompletionIntCount           : 10;  /* Number of completions to collect prior to CpltQ Int*/
    U32     Reserved10_15                       :  6;
    U32     SasCoreCompletionIntTimer           : 10;  /* Number of microseconds to hold compltion prior to CpltQ Int */
    U32     Reserved26_31                       :  6;
} IOC_CPLTQ_INT_CONFIG;

/*****************************************************************************
*
*  Manufacturing Page 10 Config Page
*   This is an IR MFG Page which will be used by firmware to
*   determine policies to apply based on OEM requirements
*
*   GenericFlags will be defined in this document as all OEMs will have access
*   to these flags.
*
*   OEMSpecificFlags will not be available in this document as these flags are
*   specific to each OEM, and the information must only be provided to the
*   specific OEM for which the values apply.
*
*****************************************************************************/

/* OEM Identifiers */
#define IOC_MFG10_OEM_ID_INVALID                   (0x00000000)
#define IOC_MFG10_OEM_ID_DELL                      (0x00000001)
#define IOC_MFG10_OEM_ID_FSC                       (0x00000002)
#define IOC_MFG10_OEM_ID_SUN                       (0x00000003)
#define IOC_MFG10_OEM_ID_IBM                       (0x00000004)
#define IOC_MFG10_OEM_ID_LSI                       (0x000000FF)

/* GENERIC Flags 0*/
#define IOC_MFG10_GF0_OCE_DISABLED                  (0x00000001)
#define IOC_MFG10_GF0_R1E_DRIVE_COUNT               (0x00000002)
#define IOC_MFG10_GF0_R10_DISPLAY                   (0x00000004)
#define IOC_MFG10_GF0_SSD_DATA_SCRUB_DISABLE        (0x00000008)
#define IOC_MFG10_GF0_SINGLE_DRIVE_R0               (0x00000010)
#define IOC_MFG10_GF0_MULTIPATHING                  (0x00000020)
#define IOC_MFG10_GF0_FAST_BGI_ENABLED              (0x00000040)
#define IOC_MFG10_GF0_CONSISTENCY_CHECKING_DISABLED (0x00000080)

/* OEM Specific Flags will come from OEM specific header files */

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_10
{
    MPI2_CONFIG_PAGE_HEADER Header;                                 /* 00h */
    U8                      OEMIdentifier;                          /* 04h */
    U8                      Reserved1;                              /* 05h */
    U16                     Reserved2;                              /* 08h */
    U32                     Reserved3;                              /* 0Ch */
    U32                     GenericFlags0;                          /* 10h */
    U32                     GenericFlags1;                          /* 14h */
    U32                     Reserved4;                              /* 18h */
    U32                     OEMSpecificFlags0;                      /* 1Ch */
    U32                     OEMSpecificFlags1;                      /* 20h */
    U32                     Reserved5[18];                          /* 24h - 60h*/
} IOC_CONFIG_PAGE_MFG_PAGE_10, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_10;

/* Miscellanous settings that do not belong elsewhere */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_11
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                                     /* 0x00 */

    U8                                  FlashTime;                                                  /* 0x04 */
    U8                                  NVTime;                                                     /* 0x05 */
    U16                                 Flag;                                                       /* 0x06 */

    U8                                  RFIoTimeout;    /* Reduced Functionality Timeout */         /* 0x08 */
    U8                                  EEDPTagMode;    /* bits 0-1 initiator, bits 4-5 target         0x09 */
    U8                                  AWTValue;       /* its only 4 bits, but easier to define as 8 for now 0x0A */
    U8                                  Reserved1;                                                  /* 0x0B */

    U8                                  MaxCmdFrames;   /* 0=F/W Default; 31 max */                 /* 0x0C */
    U8                                  Reserved2;                                                  /* 0x0D */
    U16                                 AddlFlags;                                                  /* 0x0E */

    U32                                 SysRefClk;      /* System Reference Clock in Hz */          /* 0x10 */

    U32                                 Reserved3;                                                  /* 0x14 */
    /*
    * The following are the SOD settings for the ring buffer masks
    */
    U64                                 IopDebugEnable;                                             /* 0x18 */
    U64                                 PlDebugEnable;                                              /* 0x20 */
    U64                                 IrDebugEnable;                                              /* 0x28 */

} IOC_CONFIG_PAGE_MFG_PAGE_11, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_11;

/* These allow people to enable usage of various slow or fast memories that might be available */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_NVSRAM             (0x0002)    /* 1: Use NVSRAM if found.            0: Don't detect and use NVSRAM */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_DDR                (0x0004)    /* 1: Use DDR if found.               0: Don't detect and use DDR    */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_HCDW               (0x0008)    /* 1: Use HCDW for DATA if enabled.   0: Don't use HCDW for DATA     */
/* NOTE: If running from HCDW the code must be copied to the bottom of the HCDW initially so the choice is XMEM or HCDW bottom      */
#define IOC_CFG_MFG_PG11_FLAG_XMEMCOPY                  (0x0010)    /* 1: Copy code from source to memory 0: Execute from source         */

/* Turn off the check for the broken AAMUX */
#define IOC_CFG_MFG_PG11_FLAG_DISABLE_LSI_AAMUX_CHECK   (0x0020)

/* Enable RAID Accelerator and allocate memory at SOD. */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_RAID_ACCELERATOR   (0x0040)    /* 1: Enable RAID Accelerator     0: RAID Accelerator Disabled */

/* This is a special mode that MPI 2.0 devices may use */
#define IOC_CFG_MFG_PG11_FLAG_ROL                       (0x0080)    /* 1: FMU uses ROL Mode           0: FMU uses normal MPI 2.0 mode */

/* The following are special modes that influence route table programming */
/* Enable Simplified Routing */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_SIMPL_ROUTING      (0x0100)
/* Enable Wide Port Optimization */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_WIDE_PORT_OPT      (0x0200)
/* Default value of iop direct print flag - set to 1 to debug resets with UART */
#define IOC_CFG_MFG_PG11_FLAG_DEFAULT_IOP_DIR_PRINT     (0x0400)
/* Prevent host from issuing adapter reset on a FAULT */
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_MASK          (0x1800)
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_ON_FAULT_OFF  (0x0000)    /* This is OFF */
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_ON_FAULT      (0x0800)    /* This halts with a FAULT even if due to adapter reset IRQ */
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_IGNORE_RESET  (0x1000)    /* This is a flag for the the next one */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_INITIAL_AWT        (0x4000)    /* Enable configurable initial AWT value */
#define IOC_CFG_MFG_PG11_FLAG_ENABLE_L3_OCM_IS_FAST     (0x8000)    /* When L3 is used as memory, consider it 1:fast 0:slow */

/* mask for AWT value to get least significant 4 bits */
#define IOC_CFG_MFG_PG11_AWT_VALUE_MASK                 (0x0f)
/*
 * The following may be used to block reset after a fault not caused by adapter reset
 * This is necessary due to the unexptected IRQ caused by the 10ms hw delay after adapter reset assertion
 */
#define IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_ON_FAULT_NOAR (IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_ON_FAULT|IOC_CFG_MFG_PG11_FLAG_BLOCK_RESET_IGNORE_RESET)

/* EEDP Tag mode settings for intiator and target */
#define IOC_CFG_MFG_PG11_EEDP_MODE_INIT_0       (0x00)
#define IOC_CFG_MFG_PG11_EEDP_MODE_INIT_1       (0x01)
#define IOC_CFG_MFG_PG11_EEDP_MODE_INIT_2       (0x02)
#define IOC_CFG_MFG_PG11_EEDP_MODE_TARG_0       (0x00)
#define IOC_CFG_MFG_PG11_EEDP_MODE_TARG_1       (0x10)
#define IOC_CFG_MFG_PG11_EEDP_MODE_TARG_2       (0x20)
#define IOC_CFG_MFG_PG11_EEDP_MODE_MASK         (0x03)
#define IOC_CFG_MFG_PG11_EEDP_MODE_TARG_SHIFT   (4)

/* Mfg Page 11 AddlFlags */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_EXTEND_2108_CREDIT           (0x0001)    /* Bit  0 Allows the 2108 to have credit > 1024 (not recommended) */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_SEND_EXECUTE_DEV_DIAG_CMD    (0x0002)    /* Bit  1 Send ATA Execute Device Diagnostic cmd during SATA init */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_USE_DEVICE_SLOT_NUM          (0x0008)    /* Bit  3 Use the Connector Index instead of the Device Slot Number */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_FEATURE1                     (0x0010)    /* Bit  4 Reserved for a feature */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_FEATURE2                     (0x0020)    /* Bit  5 Reserved for a feature */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_PCIE_RELAXED_ORDERING        (0x2000)    /* Bit 13 */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_RESERVED_14                  (0x4000)    /* Bit 14 Formerly BT1680 Detect Only on SATA */
#define IOC_CFG_MFG_PG11_ADDLFLAGS_RESERVED_15                  (0x8000)    /* Bit 15 Formerly BT1680 Detect Only on SAS  */



/*****************************************************************************
*
*  Manufacturing Page 12 Config Page
*
*****************************************************************************/

/*
 * Leave this define set to one and check Header.PageLength at runtime.
 */

#ifndef IOC_MAN_PAGE_12_SGPIO_INFO_ENTRIES
#define IOC_MAN_PAGE_12_SGPIO_INFO_ENTRIES         (1)
#endif

/* Defines for the Flags field */
#define IOC_CFG_MFG_12_FLAGS_SGPIO_DRIVE_PRESENCE_CHECK         (0x00000800) /* Check Drive presence to set Fault LED */
#define IOC_CFG_MFG_12_FLAGS_SGPIO_SFF8485_ACT_INVERT           (0x00000400)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_DISABLE_GROUP_ID             (0x00000200)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_DISABLE_PORT_ACT_LED         (0x00000100)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_USE_SLOT_INFO                (0x00000080)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_DATA_IN_LATCH_CTRL           (0x00000040)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_GP_MODE_CTRL                 (0x00000020)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_SLOAD_MODE_CTRL              (0x00000010)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_CLK_FILTER_ENABLE            (0x00000008)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_CTRL_OD_ENABLE               (0x00000004)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_CTRL_DOUT_OD_ENABLE          (0x00000002)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_MODE_VU                      (0x00000001)
#define IOC_CFG_MFG_12_FLAGS_SGPIO_MODE_SFF8485                 (0x00000000)


typedef struct _SGPIO_CONFIG_INFO
{
    U8                                  DeviceCount;                                       /* 00h */
    U8                                  Reserved1[3];                                      /* 01h */
    U32                                 Reserevd2;                                         /* 04h */
    U32                                 PhyInfoOrder[8];                                   /* 08h */
} SGPIO_CONFIG_INFO, *PTR_SGPIO_CONFIG_INFO;

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_12
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                            /* 00h */
    U32                                 Flags;                                             /* 04h */
    U32                                 Reserved1;                                         /* 08h */
    U32                                 Reserved2;                                         /* 0Ch */
    U32                                 SGPIOCfg1;                                         /* 0Dh */
    U8                                  NumSGPIO;                                          /* 0Eh */
    U8                                  SGPIOType;                                         /* 10h */
    U16                                 ClkDivide;                                         /* 14h */
    U32                                 DefaultTxCtrl;                                     /* 18h */
    U32                                 CustomBlinkPattern0;                               /* 1Ch */
    U32                                 CustomBlinkPattern1;                               /* 20h */
    U32                                 CustomBlinkPattern2;                               /* 24h */
    U32                                 CustomBlinkPattern3;                               /* 28h */
    U32                                 SlowBlinkPattern0;                                 /* 2Ch */
    U32                                 SlowBlinkPattern1;                                 /* 30h */
    U32                                 Reserved3;                                         /* 34h */
    U32                                 Reserved4;                                         /* 38h */
    SGPIO_CONFIG_INFO                   SGPIOInfo[IOC_MAN_PAGE_12_SGPIO_INFO_ENTRIES];     /* 3Ch */
} IOC_CONFIG_PAGE_MFG_PAGE_12, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_12;

#define IOC_CONFIG_PAGE_MFG12_SGPIO_DISABLE (0)

/* SGPIO Type Codes */
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_NONE                 (0)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_HP                   (1)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_RSA_GENERIC          (4)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_RSA_FSC              (5)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_GIGABYTE             (6)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_HITACHI              (7)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_SUN                  (8)
#define IOC_CONFIG_PAGE_MFG12_SGPIO_TYPE_IBM                  (9)

/*****************************************************************************
*
*  Manufacturing Page 13 Config Page
*
*****************************************************************************/

/*
 * Leave this define set to one and check Header.PageLength at runtime.
 */

#ifndef IOC_MAN_PAGE_13_SGPIO_ENTRIES
#define IOC_MAN_PAGE_13_SGPIO_ENTRIES         (1)
#endif

typedef struct _SGPIO_TRANSLATION_DATA
{
    U32                                 Mask;                                              /* 00h */
    U32                                 SlotStatus;                                        /* 04h */
    U8                                  TxControl[4];                                      /* 08h */
} SGPIO_TRANSLATION_DATA, *PTR_SGPIO_TRANSLATION_DATA;

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_13
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                            /* 00h */
    U8                                  NumSgpioEntries;                                   /* 04h */
    U8                                  Reserved0;                                         /* 05h */
    U16                                 Reserved1;                                         /* 06h */
    U32                                 Reserved2;                                         /* 08h */
    SGPIO_TRANSLATION_DATA              SGPIOData[IOC_MAN_PAGE_13_SGPIO_ENTRIES];          /* 0Ch */
} IOC_CONFIG_PAGE_MFG_PAGE_13, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_13;


/* Definition of SGPIO Transmit control Type in TxControl */
#define IOC_CFG_MFG_13_SGPIO_TXCONTROL_TYPE_MASK            (0x30)
#define IOC_CFG_MFG_13_SGPIO_TXCONTROL_TYPE_NONE            (0x00)
#define IOC_CFG_MFG_13_SGPIO_TXCONTROL_TYPE_ACTIVITY        (0x10)
#define IOC_CFG_MFG_13_SGPIO_TXCONTROL_TYPE_LOCATE          (0x20)
#define IOC_CFG_MFG_13_SGPIO_TXCONTROL_TYPE_ERROR           (0x30)

/*
 * Definiton of SGPIO Tranmit control bit encodings for Activity, Locate
 * and Error LED's in SFF-8485 mode.
 */

/* Activity */
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_SHIFT                     (5)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_MASK                      (0x07)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_DISABLE                   (0x00)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_ENABLE                    (0x01)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_BLINK_A_1ST_HALF          (0x02)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_BLINK_A_2ND_HALF          (0x03)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_DEASSERT_ACTIVITY         (0x04)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_ASSERTED_ACTIVITY         (0x05)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_BLINK_B_1ST_HALF          (0x06)
#define IOC_CFG_MFG_13_SFF_SGPIO_ACT_BLINK_B_2ND_HALF          (0x07)

/* Locate */
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_SHIFT                     (3)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_MASK                      (0x03)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_DISABLE                   (0x00)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_ENABLE                    (0x01)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_BLINK_A_1ST_HALF          (0x02)
#define IOC_CFG_MFG_13_SFF_SGPIO_LOC_BLINK_A_2ND_HALF          (0x03)

/* Error */
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_SHIFT                     (0)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_MASK                      (0x07)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_DISABLE                   (0x00)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_ENABLE                    (0x01)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_BLINK_A_1ST_HALF          (0x02)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_BLINK_A_2ND_HALF          (0x03)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_BLINK_B_1ST_HALF          (0x06)
#define IOC_CFG_MFG_13_SFF_SGPIO_ERR_BLINK_B_2ND_HALF          (0x07)

/* Vendor Unique Mode */
/*
 * Definiton of SGPIO Transmit control bit encoding for Activity, Error
 * and Locate LEDs in Vendor Unique mode.
 */

/* Error */
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_SHIFT                     (5)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_MASK                      (0x07)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_FORCE_OFF                 (0x00)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_FORCE_ON                  (0x01)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_PATTERN_GEN0              (0x02)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_PATTERN_GEN1              (0x03)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_PATTERN_GEN2              (0x06)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_PATTERN_GEN3              (0x07)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_PATTERN_GEN3_90           (0x04)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_PATTERN_GEN3_180          (0x05)

/* Note: PATTERN_GEN3 (_90, _180) patterns and SLOW_PATTERN (0,1)
 are mutually exclusive, only one of them can be selected */
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_SLOW_PATTERN0             (0x04)
#define IOC_CFG_MFG_13_VU_SGPIO_ERR_SLOW_PATTERN1             (0x05)

/* Locate */
#define IOC_CFG_MFG_13_VU_SGPIO_LOC_SHIFT                     (2)
#define IOC_CFG_MFG_13_VU_SGPIO_LOC_MASK                      (0x07)
#define IOC_CFG_MFG_13_VU_SGPIO_LOC_FORCE_OFF                 (0x00)
#define IOC_CFG_MFG_13_VU_SGPIO_LOC_FORCE_ON                  (0x01)
#define IOC_CFG_MFG_13_VU_SGPIO_LOC_PATTERN_GEN0              (0x02)
#define IOC_CFG_MFG_13_VU_SGPIO_LOC_PATTERN_GEN2              (0x03)

/* Activity */
#define IOC_CFG_MFG_13_VU_SGPIO_ACT_SHIFT                     (0)
#define IOC_CFG_MFG_13_VU_SGPIO_ACT_MASK                      (0x03)
#define IOC_CFG_MFG_13_VU_SGPIO_ACT_FORCE_OFF                 (0x00)
#define IOC_CFG_MFG_13_VU_SGPIO_ACT_FORCE_ON                  (0x01)
#define IOC_CFG_MFG_13_VU_SGPIO_ACT_INVERTED_ACTIVITY         (0x04)
#define IOC_CFG_MFG_13_VU_SGPIO_ACT_ASSERTED_ACTIVITY         (0x05)


/*****************************************************************************
*
*  Manufacturing Page 14 Config Page
*
*****************************************************************************/

/*
 * Leave this define set to one and check Header.PageLength at runtime.
 */

#ifndef IOC_MAN_PAGE_14_SEP_ENTRIES
#define IOC_MAN_PAGE_14_SEP_ENTRIES         (1)
#endif

/* Max Direct Attached SEP supported */
#define MAX_NUM_DA_SEP                      (6)

typedef struct _SEP_TRANSLATION_DATA
{
    U32                                 Mask;                                              /* 00h */
    U32                                 SlotStatus;                                        /* 04h */
    U32                                 SESValue;                                          /* 08h */
} SEP_TRANSLATION_DATA, *PTR_SEP_TRANSLATION_DATA;

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_14
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                            /* 00h */
    U8                                  NumSepEntries;                                     /* 04h */
    U8                                  SEPType;                                           /* 05h */
    U16                                 Reserved1;                                         /* 06h */
    U32                                 Reserved2;                                         /* 08h */
    SEP_TRANSLATION_DATA                SEPData[IOC_MAN_PAGE_14_SEP_ENTRIES];              /* 0Ch */
} IOC_CONFIG_PAGE_MFG_PAGE_14, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_14;

/* SEP Type defines  */
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_NONE                 (0)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_HP                   (1)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_RSA_GENERIC          (2)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_DELL                 (3)

#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA                 (0x80)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA_VITESSE         (IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA | 0x10)
#define IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA_IBM             (IOC_CONFIG_PAGE_MFG14_SEP_TYPE_SATA | 0x18)

#ifndef IOC_CONFIG_PAGE_VENDOR_WORDS
#define IOC_CONFIG_PAGE_VENDOR_WORDS  (1)
#endif


typedef struct _IOPI_SPD_DDR2
{
    U8 SpdDataBytes;              //  0 - 0x00 number of bytes in this SPD structure (0-255)
    U8 SizeSerialMemory;          //  1 - 0x01 encoded value = #serial memory data bytes Size==2^N bytes
    U8 FundamentalMemoryType;     //  2 - 0x02 memory type IOPI_SPD_DDR2_FUND_MEMORY_TYPE_* constant
    U8 NumberOfRowAddresses;      //  3 - 0x03 Number of RAS lines (bank 1=hi nibble, bank 2 = low nibble )
    U8 NumberOfColumnAddresses;   //  4 - 0x04 Number of CAS lines (bank 1=hi nibble, bank 2 = low nibble )
    U8 NumberOfDimmRanks;         //  5 - 0x05 Number of DIMM Ranks see IOPI_SPD_DDR_DIMM_RANK_*
    U8 ModuleWidthInBitsLSB;      //  6 - 0x06 Module Data Width of DIMM in number of bits
    U8 ModuleWidthInBitsMSB;      //  7 - 0x07 Module Data Width continued (if > 16 bytes (256bits), else 0)
    U8 VoltageInterfaceStd;       //  8 - 0x08 IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_* constant
    U8 SdramCycleTime;            //  9 - 0x09 Tclk 1/10 ns BCD (LSN.MSN ns) (N=nibble)
    U8 SdramAccessFromClock;      // 10 - 0x0A Tac 1/100ns BCD  (0.LSN MSN ns) (N=nibble)
    U8 DimmConfigurationType;     // 11 - 0x0B IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_* constant
    U8 RefreshRateAndType;        // 12 - 0x0C IOPI_SPD_DDR2_DIMM_REFRESH_RATE_* constant
    U8 PrimarySdramWidth;         // 13 - 0x0D Width of individual memory types (x4, x8, etc..) Module width 0x72 SDRAM Width 0x9 bytes (MSB bank2 2x)
    U8 ErrorCheckingSdramWidth;   // 14 - 0x0E Number of bytes error is checked in */
    U8 Reserved15;                // 15 - 0x0F Reserved in JEDEC
    U8 BurstLengthsSupported;     // 16 - 0x10 Burst Lengths supported IOPI_SPD_DDR2_BURST_LENGTH_SUPPORTED_*
    U8 NumberOfBanksEachChip;     // 17 - 0x11 (1-255)
    U8 CasLatencySupported;       // 18 - 0x12 each bit represents SDRAM (1 + N * 0.5) supported
    U8 DimmMechanicalChar;        // 19 - 0x13 Dimm Mechanical Characteristics
    U8 DimmType;                  // 20 - 0x14 Unbuffer Differential:0x20 - Registered DDR Dimmm:0x26 IOPI_SPD_DDR2_DIMM_TYPE_*
    U8 ModuleAttributes;          // 21 - 0x15 SDRAM Module Attributes
    U8 DeviceAttributes;          // 22 - 0x16 See IOPI_SPD_DDR2_DEV_ATTR_*
    U8 CycleTime2ndHighest;       // 23 - 0x17 cycle time at reduced CAS latency (X-1)  (LSN.MSN ns) (N=nibble)
    U8 AccessFromClock2ndHighest; // 24 - 0x18 access time at reduced CAS latency (X-1) (0.LSN MSN ns) (N=nibble)
    U8 CycleTime3rdHighest;       // 25 - 0x19 cycle time at reduced CAS latency (X-2)  (LSN.MSN ns) (N=nibble)
    U8 AccessFromClock3rdHighest; // 26 - 0x1A access time at reduced CAS latency (X-2) (0.LSN MSN ns) (N=nibble)
    U8 tRP;                       // 27 - 0x1B Min Row Precharge Time N/4 ns
    U8 tRRD;                      // 28 - 0x1C Min Row Active to Row Active Delay N/4 ns
    U8 tRCD;                      // 29 - 0x1D Min RAS to CAS delay N/4 ns
    U8 tRAS;                      // 30 - 0x1E Min Active to Precharge Time (ns)
    U8 ModuleRankDensity;         // 31 - 0x1F
    U8 tIS;                       // 32 - 0x20 Address and Command Input Setup Time (0.LSN MSN ns)
    U8 tIH;                       // 33 - 0x21 Address and command Hold Time After Clock (0.LSN MSN ns)
    U8 tDS;                       // 34 - 0x22 Input Setup Time before Data Strobe Before Clock (0.LSN MSN ns)
    U8 tDH;                       // 35 - 0x23 Address and Command HOld Time AFter Clock (0.LSN MSN ns)
    U8 tWR;                       // 36 - 0x24 Write Recovery Time
    U8 tWTR;                      // 37 - 0x25 Internal Write To Read Command Delay (N/4 ns)
    U8 tRTP;                      // 38 - 0x26 Internal Read to Precharge command delay (N/4 ns)
    U8 MemAnalProbeChar;          // 39 - 0x27 Memory analysis Probe Characteristics
    U8 ExtensiontRCtRFC;          // 40 - 0x28 Extension of Byte 41 tRC and Byte 42 tRFC
                                  //           tRFC=bit0*256ns+(bit1)?(Bits[2:3]+1)/2*0.25ns:Bits[2:3]*0.33ns
                                  //           tRC=(bit4)?(Bits[5:6]+1)/2*0.25ns:Bits[5:6]*0.33ns
    U8 tRC;                       // 41 - 0x29 Min Active to Active Auto Refresh Time (ns)
    U8 tRFC;                      // 42 - 0x2A Min Refresh to Active Refresh Command Period (ns)
    U8 tCKmax;                    // 43 - 0x2B Min Device Cycle Time (N/4 ns)
    U8 tDQSQ;                     // 44 - 0x2C Min Skew Between DQS and DQ (N/100 ns)
    U8 tQHS;                      // 45 - 0x2D Max Read Data Hold Skew Factor (0.LSN MSN ns)
    U8 tPllRelock;                // 46 - 0x2E PLL Relock Time
    U8 ThermalCharacteristics[15];// 47:61 0x2F  - Misc Thermal Characteristics
    U8 Revision;                  // 62 - 0x3E SPD Revision Code BCD (MSN . LSN)
    U8 Checksum;                  // 63 - 0x3F Simple Sum Checksum for 0 to 62 (MSB of sum(0:62))
    /* Note this must be a multiple of 4 bytes in length! */
} IOPI_SPD_DDR2, * PTR_IOPI_SPD_DDR2;

#define IOPI_SPD_DDR_DIMM_RANK_NUM_RANKS_MASK               (0x07)  /* Number of RAnks is N+1 */
#define IOPI_SPD_DDR_DIMM_RANK_NUM_RANKS_CARD_ON_CARD       (0x08)
#define IOPI_SPD_DDR_DIMM_RANK_NUM_RANKS_DRAM_STACK         (0x10)  /* 0x10 stack, else planar */
#define IOPI_SPD_DDR_DIMM_RANK_NUM_RANKS_MOD_HEIGHT_MASK    (0xE0)

#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_RESERVED             0
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_FPM_DRAM             1
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_EDO                  2
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_PIPEL_NIB            3
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_SDRAM                4
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_ROM                  5
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_SGRAM_DDR            6
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_SDRAM_DDR            7
#define IOPI_SPD_DDR2_FUND_MEMORY_TYPE_DDR2_SDRAM           8

#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_5VTTL           0
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_LVTTL           1
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_HSTL_1P5V       2
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_SSTL_3P3V       3
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_SSTL_2P5V       4
#define IOPI_SPD_DDR2_VOLTAGE_INTERFACE_STD_SSTL_1P8V       5

#define IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_NONE          0
#define IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_PARITY        1
#define IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_ECC           2
#define IOPI_SPD_DDR2_DIMM_CONFIGURATION_TYPE_ECC_ADDR_PARITY_DIMM (0x06)    // Address command Parity with ECC

#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_SELF_FLAG           0x80
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_MASK                0x7F
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_NORMAL              0x00    /*  15.625 us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_REDUCE_QUARTER      0x01    /*   3.9   us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_REDUCE_HALF         0x02    /*   0.7   us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_MULT_TWO            0x03    /*  31.3   us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_MULT_FOUR           0x04    /*  62.5   us */
#define IOPI_SPD_DDR2_DIMM_REFRESH_RATE_MULT_EIGHT          0x05    /* 125     us */

#define IOPI_SPD_DDR2_BURST_LENGTH_SUPPORTED_4              0x04
#define IOPI_SPD_DDR2_BURST_LENGTH_SUPPORTED_8              0x08

#define IOPI_SPD_DDR2_DIMM_TYPE_UNDEFINED                   0x00
#define IOPI_SPD_DDR2_DIMM_TYPE_RDIMM                       0x01
#define IOPI_SPD_DDR2_DIMM_TYPE_UDIMM                       0x02
#define IOPI_SPD_DDR2_DIMM_TYPE_SODIMM                      0x04
#define IOPI_SPD_DDR2_DIMM_TYPE_MICRO_DIMM                  0x08
#define IOPI_SPD_DDR2_DIMM_TYPE_MINI_RDIMM                  0x10
#define IOPI_SPD_DDR2_DIMM_TYPE_MINI_UDIMM                  0x20

#define IOPI_SPD_DDR2_DIMM_ATTR_NUM_ACTIVE_REGS_MASK        0x03
#define IOPI_SPD_DDR2_DIMM_ATTR_NUM_PLLS_MASK               0x0E
#define IOPI_SPD_DDR2_DIMM_ATTR_FET_SW_EXT_EN               0x10
#define IOPI_SPD_DDR2_DIMM_ATTR_TBD5                        0x20
#define IOPI_SPD_DDR2_DIMM_ATTR_ANALSYS_PROBE_INST          0x40
#define IOPI_SPD_DDR2_DIMM_ATTR_TBD7                        0x80

#define IOPI_SPD_DDR2_DEV_ATTR_WEAK_DRIV_SUP                0x01
#define IOPI_SPD_DDR2_DEV_ATTR_50_OHM_OUT_SUP               0x02
#define IOPI_SPD_DDR2_DEV_ATTR_PASR_SUP                     0x04
#define IOPI_SPD_DDR2_DEV_ATTR_RESERVED                     0xF1

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_16
{
    MPI2_CONFIG_PAGE_HEADER             Header;                 /* 0x00 */
    U32                                 Flags;                  /* 0x04 */
    U32                                 MaxFirmwareDDRSize;     /* 0x08 - Maximum Firmware Owned DDR in bytes */
    U8                                  DDR3_SPD[256];          /* To IOP, this is just an array of bytes.  The ddr3 library will cast it to
                                                                  it's own SPD structure for use */
} IOC_CONFIG_PAGE_MFG_PAGE_16, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_16;

#define IOC_CONFIG_PAGE_MFG16_FLAGS_MFG_SPD     (0x00000001)    /* Use IOPI_SPD_DDR2 structure for SPD data */
#define IOC_CONFIG_PAGE_MFG16_FLAGS_IIC_SPD     (0x00000002)    /* Retrieve SPD information from IIC based on ISTWI type (Mfg Page 8) */
#define IOC_CONFIG_PAGE_MFG16_FLAGS_TRAIN_MASK  (0x0000000F)    /* Mask for training modes */
#define IOC_CONFIG_PAGE_MFG16_FLAGS_RSV_MASK    (0xFFFFFFF0)

/*****************************************************************************
*
*  Manufacturing Page 17 Config Page
*
*****************************************************************************/

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_17
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                 /* 0x00 */
    U8                                  RetryMode;                              /* 0x04 */
    U8                                  SMPTimeout;
    U16                                 Reserved1;
    U32                                 Reserved2;                              /* 0x08 */
    U32                                 Flags;                                  /* 0x0C */
    U32                                 Reserved3;                              /* 0x10 */
    U8                                  OpenRejectRetryInterval[3];             /* 0x14 */
    U8                                  Reserved4;
    U8                                  OpenRejectRetryLmit[3];                 /* 0x18 */
    U8                                  Reserved5;
    U8                                  OpenRejectSRBInterval;                  /* 0x1C*/
    U8                                  OpenRejectSRBLimit[3];
    U8                                  OpenRejectNoDestInterval[3];            /* 0x20 */
    U8                                  Reserved6;
    U8                                  OpenRejectNoDestLimit[3];               /* 0x24 */
    U8                                  Reserved7;
    U32                                 Reserved8;                              /* 0x28 */
    U32                                 Reserved9;                              /* 0x2C */

}IOC_CONFIG_PAGE_MFG_PAGE_17, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_17;

#define IOC_CONFIG_PAGE_MFG17_RETRY_MODE_ORR_TIMED                          (0x00000001)
#define IOC_CONFIG_PAGE_MFG17_RETRY_MODE_ORND_TIMED                         (0x00000002)
#define IOC_CONFIG_PAGE_MFG17_RETRY_MODE_ORSRB_TIMED                        (0x00000004)

#define IOC_CONFIG_PAGE_MFG17_FLAGS_ORCRNS_NO_RESET                         (0x00000001)
#define IOC_CONFIG_PAGE_MFG17_FLAGS_ORR_NO_RESET                            (0x00000002)

#if COMPILE_OPTION_SAS_PORT_VIRT

/*****************************************************************************
*
*  Manufacturing Page 18 Config Page
*
*****************************************************************************/

#ifndef IOC_MAN_PAGE_18_PFVP_ENTRIES
#define IOC_MAN_PAGE_18_PFVP_ENTRIES        (1)
#endif

typedef struct _MFG_PAGE_18_PFVP_ENTRY
{
    U64                                 WWID;                                   /* 0x00 */
    U64                                 DeviceName;                             /* 0x08 */
} MFG_PAGE_18_PFVP_ENTRY, *PTR_MFG_PAGE_18_PFVP_ENTRY;

typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_18
{
    MPI2_CONFIG_PAGE_HEADER             Header;                                 /* 0x00 */
    U16                                 NumVPWWIDs;                             /* 0x04 */
    U8                                  NumPFVPs;                               /* 0x06 */
    U8                                  Reserved1;                              /* 0x07 */
    U32                                 Reserved2;                              /* 0x08 */
    U32                                 Reserved3;                              /* 0x0C */
    U64                                 BaseVPWWID;                             /* 0x10 */
    MFG_PAGE_18_PFVP_ENTRY              PFVP[IOC_MAN_PAGE_18_PFVP_ENTRIES];     /* 0x18 */
}IOC_CONFIG_PAGE_MFG_PAGE_18, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_18;

#endif /* COMPILE_OPTION_SAS_PORT_VIRT */


/* Miscellanous support for Fpe */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_30
{
    MPI2_CONFIG_PAGE_HEADER Header;                                                     /* 0x00 */

    U32                     Reserved1;                                                  /* 0x04 */

    U16                     EDFBFlags;      /* EndDeviceFrameBuffering flags   */       /* 0x08 */
    U8                      MaxFCS;         /* Max # of Fast Context Switches  */       /* 0x0A */
    U8                      Reserved2;                                                  /* 0x0B*/

    U16                     EDFBMaxGroupUnload; /* Max # entries per dma group */       /* 0x0C */
    U8                      EDFBThresholdSAS;   /* Max # SAS data frame xfers  */       /* 0x0E */
    U8                      EDFBThresholdSATA;  /* Max # SATA data frame xfers */       /* 0x0F  */

    U32                     Reserved3;                                                  /* 0x10 */

    U16                     FastPathFlags;      /* Settings for FPE */                  /* 0x14 */
    U8                      RaidContextOffset;  /* Offset in dwords in SCSIio request*/ /* 0x16 */
    U8                      Reserved4;                                                  /* 0x17 */

    U16                     MaxExceptionsHandled; /* # per ExceptionQueue process */    /* 0x18 */
    U16                     MaxAsyncEventsHandled;/* # per AsycQueue process  */        /* 0x1A */

    U16                     MaxIOPCompletionsHandled; /* # per IopCompQueue process */  /* 0x1C */
    U16                     Reserved5;                                                  /* 0x1E */

    U32                     Reserved6[2];                                               /* 0x20 */

} IOC_CONFIG_PAGE_MFG_PAGE_30, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_30;

#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_FCS_DISABLED   (0x0002)
#define IOC_CONFIG_PAGE_MFG30_EDFB_FLAGS_EDFB_DISABLED  (0x0001)
#define IOC_CONFIG_PAGE_MFG30_FASTPATH_FLAGS_FPE_TIMER_ENABLED   (0x0002)
#define IOC_CONFIG_PAGE_MFG30_FASTPATH_FLAGS_FP_MESSAGING_MODE   (0x0001)

#ifndef IOC_MAN_PAGE_31_NUM_UARTS
#define IOC_MAN_PAGE_31_NUM_UARTS 2
#endif

/* UART settings */
typedef struct _IOC_CONFIG_PAGE_MFG_PAGE_31
{
    MPI2_CONFIG_PAGE_HEADER Header;                                                     /* 0x00 */

    U32                     Reserved1[4];                                               /* 0x04 */
    U8                      NumUarts;       /* # UartSettings structures  */            /* 0x14 */
    U8                      Reserved2[3];                                               /* 0x15 */
    U32                     ExtUartClk;     /* External Uart Clock in Hz    */          /* 0x18 */

    /* Uart BR & Flags  */
    IOC_CONFIG_UART_SETTINGS            UartSettings[IOC_MAN_PAGE_31_NUM_UARTS];       /* 0x1C */

} IOC_CONFIG_PAGE_MFG_PAGE_31, *PTR_IOC_CONFIG_PAGE_MFG_PAGE_31;



/*****************************************************************************
*
*  Extended Manufacturing Page 40/41 Config Page
*
*****************************************************************************/

#ifndef IOC_NUM_PHYS
#define IOC_NUM_PHYS                    (8)
#endif

/* Phy settings */
typedef struct _IOC_CONFIG_PAGE_EXT_MFG_PAGE_40
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                                         /* 00h */
    U8                                  NumPhys;                                        /* 08h */
    U8                                  Reserved1[3];
    U64                                 Phy[IOC_NUM_PHYS];                              /* 0Ch */
} IOC_CONFIG_PAGE_EXT_MFG_PAGE_40, *PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_40;


/* Definition of phy group in Phy*/
#define IOC_PHY_GROUP_MASK                      (0x000000FF)    /* Bit 0~7, Specify phy group to use this phy*/
#define IOC_PHY_GROUP_0                         (0x00000000)
#define IOC_PHY_GROUP_1                         (0x00000001)
#define IOC_PHY_GROUP_2                         (0x00000002)
#define IOC_PHY_GROUP_3                         (0x00000003)
#define IOC_PHY_GROUP_4                         (0x00000004)
#define IOC_PHY_GROUP_5                         (0x00000005)
#define IOC_PHY_GROUP_6                         (0x00000006)
#define IOC_PHY_GROUP_7                         (0x00000007)

/* definition of SAS SNW participation in Phy */
//#define IOC_PHY_SAS_SNW1_PARTICIPATION          (0x00000100)    /* Bit  8, For emulation only. Do not use during normal operation. */
#define IOC_PHY_SAS_SNW2_PARTICIPATION          (0x00000200)    /* Bit  9       */
#define IOC_PHY_SAS_SNW3_PARTICIPATION          (0x00000400)    /* Bit 10       */
#define IOC_PHY_SAS_SNW_PROGRAMMING_SHIFT       (8)

/* Definition of InvRxData, InvTxData in Phy */
#define IOC_PHY_INV_RX_DATA                     (0x00000800)    /* Bit 11       */
#define IOC_PHY_INV_TX_DATA                     (0x00001000)    /* Bit 12       */

#define IOC_PHY_SAS_ONLY                        (0x00002000)    /* Bit 13, Do not link up to SATA */
#define IOC_PHY_EN_SATA_PARTIAL_POW_MGMT        (0x00004000)    /* Bit 14, partial power management */
#define IOC_PHY_EN_SATA_SLUMBER_POW_MGMT        (0x00008000)    /* Bit 15, slumber power management */

/* 12.0 Gps */
#define IOC_PHY_G4_120_WITH_SSC                 (0x00010000)    /* Bit 16       */
#define IOC_PHY_G4_120_WITHOUT_SSC              (0x00020000)    /* Bit 17       */

/* 6.0 Gps */
#define IOC_PHY_G3_60_WITH_SSC                  (0x00040000)    /* Bit 18       */
#define IOC_PHY_G3_60_WITHOUT_SSC               (0x00080000)    /* Bit 19       */

/* 3.0 Gps */
#define IOC_PHY_G2_30_WITH_SSC                  (0x00100000)    /* Bit 20       */
#define IOC_PHY_G2_30_WITHOUT_SSC               (0x00200000)    /* Bit 21       */

/* 1.5 Gps, Emulation only */
//#define IOC_PHY_G1_15_WITH_SSC                  (0x00400000)    /* Bit 22       */
//#define IOC_PHY_G1_15_WITHOUT_SSC               (0x00800000)    /* Bit 23       */

#define IOC_PHY_SATA_SSC_EN                     (0x01000000)    /* Bit 24, Spread Spectrum Clocking */
#define IOC_PHY_BACK_CHANNEL_ENABLE             (0x02000000)    /* Bit 25       */

#define IOC_PHY_DATA_SETTLE_TIME_MASK           (0x7C000000)    /* Bit 26~30    */



typedef struct _IOC_PHY_GROUP
{
    U32                                 Misc[4];                /* 0x00 */
    U32                                 Sas1_OOB_Rx_Reserved;   /* 0x10 */
    U32                                 Sas1_OOB_Tx;            /* 0x14 */
    U32                                 Sas1_3G_Rx;             /* 0x18 */
    U32                                 Sas1_3G_Tx;             /* 0x1C */
    U32                                 Sas2_3G_Rx;             /* 0x20 */
    U32                                 Sas2_3G_Tx;             /* 0x24 */
    U32                                 Sas2_6G_Rx;             /* 0x28 */
    U32                                 Sas2_6G_Tx;             /* 0x2C */
    U32                                 Sas2_12G_Rx;            /* 0x30 */
    U32                                 Sas2_12G_Tx;            /* 0x34 */
    U32                                 Sata_OOB_Rx_Reserved;   /* 0x38 */
    U32                                 Sata_OOB_Tx;            /* 0x3C */
    U32                                 Sata_3G_Rx;             /* 0x40 */
    U32                                 Sata_3G_Tx;             /* 0x44 */
    U32                                 Sata_6G_Rx;             /* 0x48 */
    U32                                 Sata_6G_Tx;             /* 0x4C */
    U32                                 Sata_12G_Rx_Reserved;    /* 0x50 */
    U32                                 Sata_12G_Tx_Reserved;   /* 0x54 */
    U32                                 Optical_3G_Rx;          /* 0x58 */
    U32                                 Optical_3G_Tx;          /* 0x5C */
    U32                                 Optical_6G_Rx;          /* 0x60 */
    U32                                 Optical_6G_Tx;          /* 0x64 */
    U32                                 Optical_12G_Rx;         /* 0x68 */
    U32                                 Optical_12G_Tx;         /* 0x6C */
} IOC_PHY_GROUP, *PTR_IOC_PHY_GROUP;


/* Phy Groups */
/* The number of phy groups is the same as the number of phys. This could change in the future. */
typedef struct _IOC_CONFIG_PAGE_EXT_MFG_PAGE_41
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                                         /* 00h */
    U8                                  NumGroups;                                      /* 08h */
    U8                                  Reserved1[3];
    IOC_PHY_GROUP                       PhyGroup[IOC_NUM_PHYS];                         /* 0Ch */
} IOC_CONFIG_PAGE_EXT_MFG_PAGE_41, *PTR_IOC_CONFIG_PAGE_EXT_MFG_PAGE_41;


/* Definition of Misc in PHY_GROUP */
#define IOC_PHY_GRP_MISC_ALWAYS_WSYNC_EN            (0x00000001)    /* Bit  0, Always word synchronize */
#define IOC_PHY_GRP_MISC_TX_INVCM2                  (0x00000002)    /* Bit  1       */
#define IOC_PHY_GRP_MISC_MANUAL_SD_OPMODE_MASK      (0x000000FC)    /* Bit  2~7     */
#define IOC_PHY_GRP_MISC_MANUAL_SD_PROTOCOL_MASK    (0x00003F00)    /* Bit  8~13    */
#define IOC_PHY_GRP_MISC_MANUAL_SD_SSCTYPE_MASK     (0x0000C000)    /* Bit 14~15    */
#define IOC_PHY_GRP_MISC_RX_LOS_FILTER_RISE_MASK    (0x001F0000)    /* Bit 16~20    */
#define IOC_PHY_GRP_MISC_SD_RX_LOS_FILTER_EN        (0x00200000)    /* Bit 21       */

#define IOC_PHY_GRP_MISC_RX_LOS_FILTER_FALL_MASK    (0x1F000000)    /* Bit 24~28    */
#define IOC_PHY_GRP_MISC_MANUAL_OPMODE_EN           (0x20000000)    /* Bit 29       */
#define IOC_PHY_GRP_MISC_MANUAL_PROTOCOL_EN         (0x40000000)    /* Bit 30       */
#define IOC_PHY_GRP_MISC_MANUAL_SSC_EN              (0x80000000)    /* Bit 31       */







/****************************************************************************
*   Vendor Specific Config Pages
****************************************************************************/
typedef struct _IOC_CONFIG_PAGE_VENDOR
{
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    Header;                                            /* 0x00 */
    U32                                 VendorData[IOC_CONFIG_PAGE_VENDOR_WORDS ];         /* 0x08 */
} IOC_CONFIG_PAGE_VENDOR;

typedef union _PAGE_IOC
{
    U32                                 HeaderU32;
    MPI2_CONFIG_PAGE_HEADER             Header;
    MPI2_CONFIG_EXTENDED_PAGE_HEADER    ExtHeader;
    MPI2_CONFIG_PAGE_MAN_0              MfgPage0;
    MPI2_CONFIG_PAGE_MAN_1              MfgPage1;
    MPI2_CONFIG_PAGE_MAN_2              MfgPage2;
    //IOC_CONFIG_PAGE_MFG_PAGE_3          MfgPage3;     // obsolete
    MPI2_CONFIG_PAGE_MAN_4              MfgPage4;
    MPI2_CONFIG_PAGE_MAN_5              MfgPage5;
    IOC_CONFIG_PAGE_MFG_PAGE_6          MfgPage6;
    MPI2_CONFIG_PAGE_MAN_7              MfgPage7;
    IOC_CONFIG_PAGE_MFG_PAGE_8          MfgPage8;
    IOC_CONFIG_PAGE_MFG_PAGE_9          MfgPage9;
    IOC_CONFIG_PAGE_MFG_PAGE_10         MfgPage10;
    IOC_CONFIG_PAGE_MFG_PAGE_11         MfgPage11;
    IOC_CONFIG_PAGE_MFG_PAGE_12         MfgPage12;
    IOC_CONFIG_PAGE_MFG_PAGE_13         MfgPage13;
    IOC_CONFIG_PAGE_MFG_PAGE_14         MfgPage14;
    IOC_CONFIG_PAGE_MFG_PAGE_16         MfgPage16;
    IOC_CONFIG_PAGE_MFG_PAGE_17         MfgPage17;
#if COMPILE_OPTION_SAS_PORT_VIRT
    IOC_CONFIG_PAGE_MFG_PAGE_18         MfgPage18;
#endif /* COMPILE_OPTION_SAS_PORT_VIRT */

    IOC_CONFIG_PAGE_MFG_PAGE_30         MfgPage30;
    IOC_CONFIG_PAGE_MFG_PAGE_31         MfgPage31;
    IOC_CONFIG_PAGE_EXT_MFG_PAGE_40     ExtMfgPage40;
    IOC_CONFIG_PAGE_EXT_MFG_PAGE_41     ExtMfgPage41;

    MPI2_CONFIG_PAGE_IO_UNIT_0          IoUnitPage0;
    MPI2_CONFIG_PAGE_IO_UNIT_1          IoUnitPage1;
    MPI2_CONFIG_PAGE_IO_UNIT_3          IoUnitPage3;
    MPI2_CONFIG_PAGE_IO_UNIT_5          IoUnitPage5;
    MPI2_CONFIG_PAGE_IO_UNIT_6          IoUnitPage6;
    MPI2_CONFIG_PAGE_IO_UNIT_7          IoUnitPage7;
    MPI2_CONFIG_PAGE_IO_UNIT_8          IoUnitPage8;
#if CBB_COMPILE_OPTION_SRIOV_EN
    MPI2_CONFIG_PAGE_IO_UNIT_10         IoUnitPage10;
#endif
    MPI2_CONFIG_PAGE_IOC_0              IocPage0;
    MPI2_CONFIG_PAGE_IOC_1              IocPage1;
    MPI2_CONFIG_PAGE_IOC_7              IocPage7;
    MPI2_CONFIG_PAGE_IOC_8              IocPage8;
    MPI2_CONFIG_PAGE_BIOS_1             Bios1;
    MPI2_CONFIG_PAGE_BIOS_2             Bios2;
    MPI2_CONFIG_PAGE_BIOS_3             Bios3;
    MPI2_CONFIG_PAGE_BIOS_4             Bios4;
    MPI2_CONFIG_PAGE_SASIOUNIT_0        SasIoUnitPage0;
    MPI2_CONFIG_PAGE_SASIOUNIT_1        SasIoUnitPage1;
    MPI2_CONFIG_PAGE_SASIOUNIT_4        SasIoUnitPage4;
    MPI2_CONFIG_PAGE_SASIOUNIT_5        SasIoUnitPage5;
    MPI2_CONFIG_PAGE_SASIOUNIT_6        SasIoUnitPage6;
    MPI2_CONFIG_PAGE_SASIOUNIT_7        SasIoUnitPage7;
    MPI2_CONFIG_PAGE_SASIOUNIT_8        SasIoUnitPage8;
    MPI2_CONFIG_PAGE_SASIOUNIT16        SasIoUnitPage16;
    MPI2_CONFIG_PAGE_EXPANDER_0         SasExpanderPage0;
    MPI2_CONFIG_PAGE_EXPANDER_1         SasExpanderPage1;
    MPI2_CONFIG_PAGE_SAS_DEV_0          SasDevicePage0;
    MPI2_CONFIG_PAGE_SAS_DEV_1          SasDevicePage1;
    MPI2_CONFIG_PAGE_SAS_PHY_0          SasPhyPage0;
    MPI2_CONFIG_PAGE_SAS_PHY_1          SasPhyPage1;
    MPI2_CONFIG_PAGE_SAS_PHY_2          SasPhyPage2;
    MPI2_CONFIG_PAGE_SAS_PHY_3          SasPhyPage3;
    MPI2_CONFIG_PAGE_SAS_PHY_4          SasPhyPage4;
    MPI2_CONFIG_PAGE_SAS_ENCLOSURE_0    SasEnclPage0;
    MPI2_CONFIG_PAGE_SAS_PORT_0         SasPortPage0;

    MPI2_CONFIG_PAGE_LOG_0              Log0;
    MPI2_CONFIG_PAGE_DRIVER_MAPPING_0   DriverMapping0;

    MPI2_CONFIG_PAGE_ETHERNET_0         EthernetPage0;
    MPI2_CONFIG_PAGE_ETHERNET_1         EthernetPage1;

    IOC_CONFIG_PAGE_VENDOR              VendorPage;
#if (RAID)
    MPI2_CONFIG_PAGE_IOC_6              IocPage6;
    MPI2_CONFIG_PAGE_RAID_VOL_0         RaidVolPage0;
    MPI2_CONFIG_PAGE_RAID_VOL_1         RaidVolPage1;
    MPI2_CONFIG_PAGE_RD_PDISK_0         RaidPhysDiskPage0;
    MPI2_CONFIG_PAGE_RD_PDISK_1         RaidPhysDiskPage1;
    MPI2_CONFIG_PAGE_RAID_CONFIGURATION_0 RaidConfigPage0;
#endif /* (RAID) */

} PAGE_IOC, *PTR_PAGE_IOC;

/*
 * If any of the "IOC" pages above or pages in mpi2_cnfg.h have new
 * fields added or change in size we must increment the major field
 * of IOPI_CFGI_NVDATA_VERSION. From "How NVDATA is used":
 *   The Major version number changes when the MPT Configuration Pages have changed.
 *   Any of the following will result in a change to the Major version number:
 *      *A Configuration Page has increased in length
 *      *A new value has been added to a Configuration Page
 *      *A new Configuration Page has been created
 * Major - 16 bits, [15:8]==Phase [7:6]=sub-phase [5:0]==NVDATA change within subphase
 */
#define IOPI_CFGI_NVDATA_VERSION_MAJOR  (0x0009)    /* Major version reflects format of config pages */
#define IOPI_CFGI_NVDATA_VERSION_MINOR  (0x0000)    /* Minor version reflects specific values, used by customer */
#define IOPI_CFGI_NVDATA_VERSION        ((IOPI_CFGI_NVDATA_VERSION_MAJOR << 16)|IOPI_CFGI_NVDATA_VERSION_MINOR)

#endif /* end of file */
