/***************************************************************************
 *                                                                         *
 *                                                                         *
 *  Copyright 2003 LSI Logic Corporation.  All rights reserved.            *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiDefs.H
 *          Title:  IOP Interface Definitions
 *     Programmer:
 *  Creation Date:  April 4, 2003
 *
 *
 * Description
 * ------------
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPI_DEFS_H_INCLUDED
#define IOPI_DEFS_H_INCLUDED

#if  !defined(__LANGUAGE_ASM__)
typedef enum
{
    IOPI_SUBSYSTEM_IOP = 0,          // Force the first entry to be 0
    IOPI_SUBSYSTEM_IR,
    IOPI_SUBSYSTEM_PL,
    IOPI_SUBSYSTEM_NUM_SUBSYSTEMS    // This has to be last
} IOPI_SUBSYSTEM;

typedef enum
{
    IOPI_RESOURCE_MESSAGE_FRAME
} IOPI_RESOURCE_TYPE;
#endif /* !defined(__LANGUAGE_ASM__) */

/*
 * If the following bit is set it means this region has the following attributes:
 * EA=RA and ERPN == 0, that is you only need EA to describe a structure to hardware.
 */
#define IOPI_MEM_REGION_DMABLE ((U32)PLI_MEMORY_ATTRIBUTE_DMA_ABLE)

/*
 * The following are used as parameters for MALLOC routines as well as the IOP_MEM_REGION flags
 */
#define IOPI_MEM_RESERVED_0  ((U32)PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_RESERVED_0)
#define IOPI_MEM_FAST        ((U32)PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_REQUIRED_FAST)
#define IOPI_MEM_PREFER_FAST ((U32)PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_PREFERRED_FAST)
#define IOPI_MEM_ANY_SPEED   ((U32)PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_DONT_CARE)
#define IOPI_MEM_SLOW        ((U32)PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_SLOW)
#define IOPI_MEM_REGION_SPEED_MASK  ((U32)PLI_MEMORY_ATTRIBUTE_ACCESS_SPEED_MASK)

/*
 * If the following bit is set this memory region has caching enabled.
 * If locked data cache is supported the user MUST NOT do a generic data cache flush.
 * non-cached - no flushing necessary before hw access.
 */
#define IOPI_MEM_REGION_CACHED      ((U32)0x00000100)
#define IOPI_MEM_REGION_MAPPED      ((U32)0x00000200)
/* The following flag is set for regions that are tested and fail */
#define IOPI_MEM_REGION_TEST_FAILED ((U32)0x00000400)

/*****************************************************************************
*
*       P r i n t   S u b s y s t e m   D e f i n i t i o n s
*
*****************************************************************************/

#define IOPI_PRINT_SUBSYS_SHIFT          (14)
#define IOPI_PRINT_SUBSYS_IOP            (0 << IOPI_PRINT_SUBSYS_SHIFT)
#define IOPI_PRINT_SUBSYS_PL             (1 << IOPI_PRINT_SUBSYS_SHIFT)
#define IOPI_PRINT_SUBSYS_IR             (2 << IOPI_PRINT_SUBSYS_SHIFT)
#define IOPI_PRINT_INDEX_MASK            (0x3FFF)

/******************************************************************************
*
*      I O P   D E B U G   P R I N T   M A C R O S
*
*******************************************************************************/

/****************************************************************************/
/* Debug Print bit mask levels that make up IOP_DBG_MASK                    */
/****************************************************************************/
#define IOP_DBG_OFF             (0x0000000000000000)
#define IOP_DBG_INIT            (0x0000000100000000)
#define IOP_DBG_MSGS            (0x0000000400000000)
#define IOP_DBG_CFG_PAGE        (0x0000000800000000)
#define IOP_DBG_EXCP            (0x0000004000000000)
#define IOP_DBG_ERR             (0x0000008000000000)
#define IOP_DBG_SCSI_MSG        (0x0000010000000000)
#define IOP_DBG_DEVICE_MGMT     (0x0000020000000000)
#define IOP_DBG_IOPATH_HIGH     (0x0000100000000000)
#define IOP_DBG_IOPATH_MED      (0x0000200000000000)
#define IOP_DBG_IOPATH_LOW      (0x0000400000000000)
#define IOP_DBG_POLL            (0x0000800000000000)
#define IOP_DBG_DOORBELL        (0x0001000000000000)
#define IOP_DBG_TIMESTAMP       (0x0002000000000000)
#define IOP_DBG_ETHERNET        (0x0004000000000000)
#define IOP_DBG_PWRMGMT         (0x0200000000000000)
#define IOP_DBG_STRINGS         (0x8000000000000000) /* Include strings in debug build */
#define IOP_DBG_ENABLE          (0x6000000000000000) /* Enable debug even though no prints enabled */
#define IOP_DBG_ALL             (0xFFFFFFFFFFFFFFFF)

#if !defined(NVDATA_UTILITY)
#if (IOP_DBG_MASK && defined(BOOTLOADER) == 0)
    #define _IOP_PRINT0( Index, Pattern) if (IOP_DBG_MASK & Pattern) iopiTracePrint0(Pattern, Index | IOPI_PRINT_SUBSYS_IOP);
    #define _IOP_PRINT1( Index, Pattern, Arg1 ) if (IOP_DBG_MASK & Pattern) iopiTracePrint1(Pattern, Index | IOPI_PRINT_SUBSYS_IOP, (U32)Arg1);
    #define _IOP_PRINT2( Index, Pattern, Arg1, Arg2 ) if (IOP_DBG_MASK & Pattern) iopiTracePrint2(Pattern, Index | IOPI_PRINT_SUBSYS_IOP, (U32)Arg1, (U32)Arg2);
    #define _IOP_PRINT3( Index, Pattern, Arg1, Arg2, Arg3 ) if (IOP_DBG_MASK & Pattern) iopiTracePrint3(Pattern, Index | IOPI_PRINT_SUBSYS_IOP, (U32)Arg1, (U32)Arg2, (U32)Arg3);
    #define _IOP_PRINT4( Index, Pattern, Arg1, Arg2, Arg3, Arg4 ) if (IOP_DBG_MASK & Pattern) iopiTracePrint4(Pattern, Index | IOPI_PRINT_SUBSYS_IOP, (U32)Arg1, (U32)Arg2, (U32)Arg3, (U32)Arg4);

    #define IOP_PRINT0( Index, Pattern )                         do { \
                                                                      COMPILER_ASSERT(NUM_PRINT_PARAMS(IOP, Index) == 0); \
                                                                      _IOP_PRINT0(Index, Pattern);\
                                                                    } while(0)
    #define IOP_PRINT1( Index, Pattern, Arg1 )                   do { \
                                                                      COMPILER_ASSERT(NUM_PRINT_PARAMS(IOP, Index) == 1); \
                                                                      _IOP_PRINT1(Index, Pattern, Arg1);\
                                                                    } while(0)
    #define IOP_PRINT2( Index, Pattern, Arg1, Arg2 )             do { \
                                                                      COMPILER_ASSERT(NUM_PRINT_PARAMS(IOP, Index) == 2); \
                                                                      _IOP_PRINT2(Index, Pattern, Arg1, Arg2);\
                                                                    } while(0)
    #define IOP_PRINT3( Index, Pattern, Arg1, Arg2, Arg3 )       do { \
                                                                      COMPILER_ASSERT(NUM_PRINT_PARAMS(IOP, Index) == 3); \
                                                                      _IOP_PRINT3(Index, Pattern, Arg1, Arg2, Arg3);\
                                                                    } while(0)
    #define IOP_PRINT4( Index, Pattern, Arg1, Arg2, Arg3, Arg4 ) do { \
                                                                      COMPILER_ASSERT(NUM_PRINT_PARAMS(IOP, Index) == 4); \
                                                                      _IOP_PRINT4(Index, Pattern, Arg1, Arg2, Arg3, Arg4);\
                                                                    } while(0)

    #define IOP_TIMESTAMP() iopPrintTimestamp();
#else
    #define IOP_PRINT0( Index, Pattern)
    #define IOP_PRINT1( Index, Pattern, Arg1)
    #define IOP_PRINT2( Index, Pattern, Arg1, Arg2 )
    #define IOP_PRINT3( Index, Pattern, Arg1, Arg2, Arg3 )
    #define IOP_PRINT4( Index, Pattern, Arg1, Arg2, Arg3, Arg4 )
    #define IOP_TIMESTAMP()
#endif
#endif

#define IOPI_INVALID_MID            (0)
#define IOPI_INVALID_DEV_HANDLE     PLI_DEV_INVALID_HANDLE

#ifndef IOC_BLD_FRAME_SIZE
#define IOC_BLD_FRAME_SIZE          (128)
#endif

#define IOPI_FRAME_SIZE             IOC_BLD_FRAME_SIZE  // TODO remove when IR is converts to runtime frame size
#define IOPI_MIN_FRAME_SIZE         IOC_BLD_FRAME_SIZE
#if THUNDERBOLT
#define IOPI_MAX_FRAME_SIZE        (1008)  // mossberg limit
#else
#define IOPI_MAX_FRAME_SIZE         (768)  // limited by Nova hardware
#endif


/****************************************************************************
 *  The following are used primarily to create big-endian versions of the
 *  Footer signature from the MPI defined values.  The assembler doesn't
 *  seem to like the complicated statements required for SWAB()
 ****************************************************************************/
#define IOPI_INIT_IMAGE_SIGNATURE0_LE               (MPI2_INIT_IMAGE_SIGNATURE_BYTE_0|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_1<<8)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_2<<16)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_3<<24))
#define IOPI_INIT_IMAGE_SIGNATURE0_BE               (MPI2_INIT_IMAGE_SIGNATURE_BYTE_3|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_2<<8)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_1<<16)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_0<<24))

#define IOPI_INIT_IMAGE_SIGNATURE1_LE               (MPI2_INIT_IMAGE_SIGNATURE_BYTE_4|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_5<<8)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_6<<16)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_7<<24))
#define IOPI_INIT_IMAGE_SIGNATURE1_BE               (MPI2_INIT_IMAGE_SIGNATURE_BYTE_7|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_6<<8)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_5<<16)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_4<<24))

#define IOPI_INIT_IMAGE_SIGNATURE2_LE               (MPI2_INIT_IMAGE_SIGNATURE_BYTE_8|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_9<<8)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_A<<16)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_B<<24))
#define IOPI_INIT_IMAGE_SIGNATURE2_BE               (MPI2_INIT_IMAGE_SIGNATURE_BYTE_B|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_A<<8)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_9<<16)|(MPI2_INIT_IMAGE_SIGNATURE_BYTE_8<<24))

/*
 * The following flags are used to determine special modes for the init code
 */
#define IOPI_INIT_BOOTFLAGS_BYTE0_PCI_1_FIX        (0x01)   /* 1: Set bits for a PCI 1.0 training problem 0: normal mode */
#define IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_MASK    (0x06)
#define IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_MPT     (0x02)
#define IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_I2O     (0x04)
#define IOPI_INIT_BOOTFLAGS_BYTE0_DISABLE_EXT_TAG  (0x08)
#define IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_AUTO    (0x00)
#define IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_RSVD    (0xF0)

#if CBB_COMPILE_OPTIONS_FORCE_MPT
#if PCIE_ZIP_CHIP_WORK_AROUND
#define IOPI_FW_FOOTER_BOOTFLAGS_DEFAULT            (IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_MPT|IOPI_INIT_BOOTFLAGS_BYTE0_PCI_1_FIX)
#define IOPI_FW_FOOTER_BOOTFLAGS_BE_DEFAULT        ((IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_MPT|IOPI_INIT_BOOTFLAGS_BYTE0_PCI_1_FIX) << 24)
#else /* PCIE_ZIP_CHIP_WORK_AROUND */
#define IOPI_FW_FOOTER_BOOTFLAGS_DEFAULT           (IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_MPT)
#define IOPI_FW_FOOTER_BOOTFLAGS_BE_DEFAULT        (IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_MPT<< 24)
#endif /* PCIE_ZIP_CHIP_WORK_AROUND */
#else  /* CBB_COMPILE_OPTIONS_FORCE_MPT */
#if PCIE_ZIP_CHIP_WORK_AROUND
#define IOPI_FW_FOOTER_BOOTFLAGS_DEFAULT            (IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_AUTO|IOPI_INIT_BOOTFLAGS_BYTE0_PCI_1_FIX)
#define IOPI_FW_FOOTER_BOOTFLAGS_BE_DEFAULT        ((IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_AUTO|IOPI_INIT_BOOTFLAGS_BYTE0_PCI_1_FIX) << 24)
#else /* PCIE_ZIP_CHIP_WORK_AROUND */
#define IOPI_FW_FOOTER_BOOTFLAGS_DEFAULT           (IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_AUTO)
#define IOPI_FW_FOOTER_BOOTFLAGS_BE_DEFAULT        (IOPI_INIT_BOOTFLAGS_BYTE0_MU_FORCE_AUTO << 24)
#endif /* PCIE_ZIP_CHIP_WORK_AROUND */
#endif /* CBB_COMPILE_OPTIONS_FORCE_MPT */

#if  !defined(__LANGUAGE_ASM__)
typedef U16                         IOPI_DEV_HANDLE;
typedef U16                         IOPI_DEV_NUMBER;

#endif /* !defined(__LANGUAGE_ASM__) */


/* Constants */
/* sysDMA memoryMove() flag parameter */
#define IOPI_MEMMOVE_FLAGS_TYPE_SHIFT               (28)
#define IOPI_MEMMOVE_FLAGS_TYPE_MASK                ((U32)MPI2_SCSIIO_SGLFLAGS_TYPE_MASK      << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
#define IOPI_MEMMOVE_FLAGS_TYPE_MPI                 ((U32)MPI2_SCSIIO_SGLFLAGS_TYPE_MPI       << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
#define IOPI_MEMMOVE_FLAGS_TYPE_IEEE32              ((U32)MPI2_SCSIIO_SGLFLAGS_TYPE_IEEE32    << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
#define IOPI_MEMMOVE_FLAGS_TYPE_IEEE64              ((U32)MPI2_SCSIIO_SGLFLAGS_TYPE_IEEE64    << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
/* the following ADDR flags are ignored in IEEE mode because they are in the SGE */
#define IOPI_MEMMOVE_FLAGS_ADDR_MASK                ((U32)MPI2_SCSIIO_SGLFLAGS_ADDR_MASK      << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
#define IOPI_MEMMOVE_FLAGS_SYSTEM_ADDR              ((U32)MPI2_SCSIIO_SGLFLAGS_SYSTEM_ADDR    << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
#define IOPI_MEMMOVE_FLAGS_IOCDDR_ADDR              ((U32)MPI2_SCSIIO_SGLFLAGS_IOCDDR_ADDR    << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
#define IOPI_MEMMOVE_FLAGS_IOCPLB_ADDR              ((U32)MPI2_SCSIIO_SGLFLAGS_IOCPLB_ADDR    << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
#define IOPI_MEMMOVE_FLAGS_IOCPLBNTA_ADDR           ((U32)MPI2_SCSIIO_SGLFLAGS_IOCPLBNTA_ADDR << IOPI_MEMMOVE_FLAGS_TYPE_SHIFT)
#define IOPI_MEMMOVE_FLAGS_DIRECTION_MASK           (0x00000001)
#define IOPI_MEMMOVE_FLAGS_IOC_TO_HOST              (0) /* direction to Host */
#define IOPI_MEMMOVE_FLAGS_HOST_TO_IOC              (1) /* direction from Host to IOC */
#define IOPI_MEMMOVE_FLAGS_UNKNOWN                  (2) /* direction Unknown */

/******************************************************************************
*
*      T I M E R
*
*******************************************************************************/

/* Constants */

/* Convert timer value from Min, Sec, Sixteenths to Sixteenths */
#define IOPI_TIMER_MSS_TO_S(Minutes, Seconds, Sixteenths) ((((Minutes * 60) + Seconds) * 16) + Sixteenths)

#define IOPI_TIMER_ONE_SIXTEENTH_SECOND              IOPI_TIMER_MSS_TO_S(0, 0, 1)
#define IOPI_TIMER_ONE_EIGHTH_SECOND                 IOPI_TIMER_MSS_TO_S(0, 0, 2)
#define IOPI_TIMER_ONE_QUARTER_SECOND                IOPI_TIMER_MSS_TO_S(0, 0, 4)
#define IOPI_TIMER_ONE_HALF_SECOND                   IOPI_TIMER_MSS_TO_S(0, 0, 8)
#define IOPI_TIMER_ONE_SECOND                        IOPI_TIMER_MSS_TO_S(0, 1, 0)
#define IOPI_TIMER_TWO_SECONDS                       ( IOPI_TIMER_ONE_SECOND * 2 )
#define IOPI_TIMER_THREE_SECONDS                     ( IOPI_TIMER_ONE_SECOND * 3)
#define IOPI_TIMER_FOUR_SECONDS                      ( IOPI_TIMER_ONE_SECOND * 4)
#define IOPI_TIMER_FIVE_SECONDS                      ( IOPI_TIMER_ONE_SECOND * 5)
#define IOPI_TIMER_SIX_SECONDS                       ( IOPI_TIMER_ONE_SECOND * 6 )
#define IOPI_TIMER_SEVEN_SECONDS                     ( IOPI_TIMER_ONE_SECOND * 7 )
#define IOPI_TIMER_EIGHT_SECONDS                     ( IOPI_TIMER_ONE_SECOND * 8 )
#define IOPI_TIMER_NINE_SECONDS                      ( IOPI_TIMER_ONE_SECOND * 9 )
#define IOPI_TIMER_TEN_SECONDS                       ( IOPI_TIMER_ONE_SECOND * 10 )
#define IOPI_TIMER_FIFTEEN_SECONDS                   ( IOPI_TIMER_ONE_SECOND * 15 )
#define IOPI_TIMER_TWENTY_SECONDS                    ( IOPI_TIMER_ONE_SECOND * 20 )
#define IOPI_TIMER_TWENTY_FIVE_SECONDS               ( IOPI_TIMER_ONE_SECOND * 25 )
#define IOPI_TIMER_THIRTY_SECONDS                    ( IOPI_TIMER_ONE_SECOND * 30 )
#define IOPI_TIMER_THIRTY_FIVE_SECONDS               ( IOPI_TIMER_ONE_SECOND * 35 )
#define IOPI_TIMER_FORTY_SECONDS                     ( IOPI_TIMER_ONE_SECOND * 40 )
#define IOPI_TIMER_FORTY_FIVE_SECONDS                ( IOPI_TIMER_ONE_SECOND * 45 )
#define IOPI_TIMER_FIFTY_SECONDS                     ( IOPI_TIMER_ONE_SECOND * 50 )
#define IOPI_TIMER_FIFTY_FIVE_SECONDS                ( IOPI_TIMER_ONE_SECOND * 55 )

#define IOPI_TIMER_ONE_MINUTE                        ( IOPI_TIMER_ONE_SECOND * 60)
#define IOPI_TIMER_FIFTEEN_MINUTES                   ( IOPI_TIMER_ONE_MINUTE * 15)
#define IOPI_TIMER_THIRTY_MINUTES                    ( IOPI_TIMER_ONE_MINUTE * 30)

#define IOPI_TIMER_ONE_HOUR                          ( IOPI_TIMER_ONE_MINUTE * 60)

#if  !defined(__LANGUAGE_ASM__)
typedef U8 IOP_TIMER_HANDLE;
#endif

/* These defines are used to determine where firmware might be located */
#define IOPI_FLASH_BLOADER_START                 (0xFFFE0000)    /* -128K */
#define IOPI_FLASH_BLOADER_INC                   (0x00010000)    /* support for min 64K region/sector alignment */
#define IOPI_FLASH_BLOADER_END                   (0xFF000000)    /* -16M */

#if  !defined(__LANGUAGE_ASM__)
/* Enumerate the MPI2 FLASH region types for iopiFlashWrite and iopiFlashRead */
typedef enum
{
    IOPI_FLASH_REGION_UNUSED_ENUM            = MPI2_FLASH_REGION_UNUSED         , /* IOC FW Image in FWUpload*/
    IOPI_FLASH_REGION_FIRMWARE_ENUM          = MPI2_FLASH_REGION_FIRMWARE       ,
    IOPI_FLASH_REGION_BIOS_ENUM              = MPI2_FLASH_REGION_BIOS           ,
    IOPI_FLASH_REGION_RESERVED1_ENUM         = 3  /* Legacy */                  ,
    IOPI_FLASH_REGION_RESERVED_ENUM          = 4, /* Legacy */
    IOPI_FLASH_REGION_FIRMWARE_BACKUP_ENUM   = MPI2_FLASH_REGION_FIRMWARE_BACKUP,
    IOPI_FLASH_REGION_MFG_INFORMATION_ENUM   = MPI2_FLASH_REGION_MFG_INFORMATION,
    IOPI_FLASH_REGION_CONFIG_1_ENUM          = MPI2_FLASH_REGION_CONFIG_1       ,
    IOPI_FLASH_REGION_CONFIG_2_ENUM          = MPI2_FLASH_REGION_CONFIG_2       ,
    IOPI_FLASH_REGION_OTHER_ENUM             = MPI2_FLASH_REGION_MEGARAID       ,
    IOPI_FLASH_REGION_INIT_ENUM              = MPI2_FLASH_REGION_INIT           ,
    IOPI_FLASH_REGION_COMPLETE_ENUM          = 0xFE,
    IOPI_FLASH_REGION_INVALID_ENUM           = 0xFF
} IOPI_FLASH_REGION_TYPE;
#endif

/* These defines are used by iopiGPIORead() and iopiGPIOWrite() */
#define IOPI_ASSERT         1
#define IOPI_DEASSERT       0

/* This defines extended image max */
#define IOPI_EXT_IMAGE_TYPE_MAX                 (0x80)
#define IOPI_EXT_IMAGE_VENDOR_SPECIFIC          (0x80)

/******************************************************************************
*
*      D I A G N O S T I C   C O N S O L E   S U P P O R T
*
*******************************************************************************/
// defines used for parsing diag command tables
#define NO_ACTION              (0)
#define MAX_TABLE_DEPTH        (4)
#define MAX_ARGS               (4)

#define CLI_TOKEN              ";0," /* A string token */
#define CLI_TOKEN_ID           (0)

#define CLI_ADDRESS            ";1," /* A 32-bit address */
#define CLI_ADDRESS_ID         (1)

#define CLI_VALUE              ";2," /* A 32-bit value */
#define CLI_VALUE_ID           (2)

#if  !defined(__LANGUAGE_ASM__)
#if (MEM_TEST_CFG || defined(NVDATA_UTILITY))
/* When compiling with ISO_C you must have at least one argument */
typedef void (*cliHandler)(U32 ISO_C_ARG, ...);
#else

#if defined __GNUC__
typedef void (*cliHandler)(U32 ISO_C_ARG, ...);
#else
#ifndef __TINYC__
#ifndef __WATCOMC__
typedef void (*cliHandler)(...);
#else
typedef U32 cliHandler;
#endif
#else
typedef U32 cliHandler;
#endif
#endif

#endif

typedef struct _DIAG_COMMAND_TABLE
{
    const char *cmdName;     /* string: the name of the command */
    const char *options;     /* string: the argument template */
    cliHandler func;         /* function ptr: the command handler */
    const struct _DIAG_COMMAND_TABLE *subTbl;   /* table ptr: indicates a sub-table */
    const char *shortHelp;   /* string: short help reminder */
}DIAG_COMMAND_TABLE, * PTR_DIAG_COMMAND_TABLE;

typedef const DIAG_COMMAND_TABLE CONST_DIAG_COMMAND_TABLE;

#endif /* !defined(__LANGUAGE_ASM__) */

#define IOPI_DEFS_PCIE_REV_LIB_A0           (0)
#define IOPI_DEFS_PCIE_REV_LIB_B0           (1)
#define IOPI_DEFS_PCIE_REV_LIB_B1           (2) /* NOTE: HW reads same as B.0 */
#define IOPI_DEFS_PCIE_REV_LIB_B2           (3)
#define IOPI_DEFS_PCIE_REV_LIB_B3           (4)
#define IOPI_DEFS_PCIE_REV_LIB_B4           (5)

#define IOPI_DEFS_PCIE_REV_FAL_A0           (0)
#define IOPI_DEFS_PCIE_REV_FAL_B0           (1)
#define IOPI_DEFS_PCIE_REV_FAL_B1           (2)
#define IOPI_DEFS_PCIE_REV_FAL_B2           (3)

#define IOPI_DEFS_PCIE_REV_MET_A0           (0)
#define IOPI_DEFS_PCIE_REV_MET_B0           (1)
#define IOPI_DEFS_PCIE_REV_MET_B1           (2)

#define IOPI_DEFS_PCIE_REV_TB_A0            (0)
#define IOPI_DEFS_PCIE_REV_TB_B0            (1)
#define IOPI_DEFS_PCIE_REV_TB_C0            (2)
#define IOPI_DEFS_PCIE_REV_TB_C1            (3)
#define IOPI_DEFS_PCIE_REV_TB_D0            (4)


/*******************************************/
/*     Product Specific IOC Parameters     */
/*******************************************/

#define IOC_PARAM_SET_CONTROL_GROUP     (0x01)
#define IOC_PARAM_SET_DMA_GROUP         (0x02)
#define IOC_PARAM_TYPE_SRIOV_VIS        (0x80)

#define IOC_PARAM_VALUE_SRIOV_VIS_MASK  (0x00000001)
#define IOC_PARAM_VALUE_SRIOV_VF_MASK   (0x0000FF00)
#define IOP_PARAM_VALUE_SRIOV_VF_SHIFT  (8)

#endif
