
/***************************************************************************
 *                                                                         *
 *  Copyright 2003 LSI Logic Corporation.  All rights reserved.            *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiIocLogInfo.h
 *          Title:  SAS Firmware IOP Interface IOC Log Info Definitions
 *     Programmer:
 *  Creation Date:  September 24, 2003
 *
 *
 * Description
 * ------------
 * This include file contains SAS firmware interface IOC Log Info codes
 *
 *-------------------------------------------------------------------------
 */

#ifndef IOPI_IOCLOGINFO_H_INCLUDED
#define IOPI_IOCLOGINFO_H_INCLUDED

/****************************************************************************/
/*  IOC LOGINFO defines, 0x00000000 - 0x0FFFFFFF                            */
/*  Format:                                                                 */
/*      Bits 31-28: MPI2_IOCLOGINFO_TYPE_SAS (3)                            */
/*      Bits 27-24: IOC_LOGINFO_ORIGINATOR: 0=IOP, 1=PL, 2=IR               */
/*      Bits 23-16: LOGINFO_CODE                                            */
/*      Bits 15-0:  LOGINFO_CODE Specific                                   */
/****************************************************************************/

/****************************************************************************/
/* IOC_LOGINFO_ORIGINATOR defines                                           */
/****************************************************************************/

/* See pliIocLogInfo.h */

/****************************************************************************/
/* LOGINFO_CODE defines                                                     */
/****************************************************************************/

/* See pliIocLogInfo.h */

/****************************************************************************/
/* IOP LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = IOP          */
/****************************************************************************/
#define IOP_LOGINFO_CODE_INVALID_SAS_ADDRESS            (0x00010000) /* Invalid SAS address */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE            (0x00030000) /* Invalid configuration page */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_RT         (0x00030100) /* Route table entry not found */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_PN         (0x00030200) /* Invalid page Number */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_FORM       (0x00030300) /* Invalid form in page address */
#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_DEFAULT    (0x00030700) /* Default Page not found */

#define IOP_LOGINFO_CODE_DIAG_MSG_ERROR                 (0x00040000) /* Error handling diagnostic message - ORed with diagnostic status */

#define IOP_LOGINFO_CODE_TASK_TERMINATED                (0x00050000) /* SCSI IO task terminated */

#define IOP_LOGINFO_CODE_ENCL_MGMT_READ_ACTION_ERROR    (0x00060001) /* Read action not supported for SEP message */
#define IOP_LOGINFO_CODE_ENCL_MGMT_INVALID_DEVH_ERROR   (0x00060002) /* Invalid device handle in SEP message */

#define IOP_LOGINFO_CODE_SCSI_IO_NOT_REGISTERED         (0x00070000) /* SCSI IO not registered */
#define IOP_LOGINFO_CODE_SCSI_IO_DEVICE_BLOCKED         (0x00070001) /* SCSI IO device blocked */

#define IOP_LOGINFO_CODE_VP_SAS_ADDRESS_INVALID         (0x00080001) /* Virtual port SAS address not valid */
#define IOP_LOGINFO_CODE_VP_SAS_ADDRESS_NOT_AVAILABLE   (0x00080002) /* Virtual port SAS address not available */

/*flash error*/
#define IOP_LOGINFO_CODE_FLASH_NOT_PRESENT              (0x00010101) /* Flash not present */
#define IOP_LOGINFO_CODE_FLASH_ERASE_FAILED             (0x00010102) /* Erasing flash failed */
#define IOP_LOGINFO_CODE_FLASH_WRITE_FAILED             (0x00010103) /* Writing to flash failed */

/*image error while trying to FW download*/
#define IOP_LOGINFO_CODE_IMAGE_BOOTLOADER_NOT_PRESENT   (0x00010401) /* Bootloader image not found */
#define IOP_LOGINFO_CODE_IMAGE_MAIN_CHECKSUM_FAILURE    (0x00010402) /* Main image checksum failed */
#define IOP_LOGINFO_CODE_IMAGE_EXT_CHECKSUM_FAILURE     (0x00010403) /* Extended image checksum failed */
#define IOP_LOGINFO_CODE_IMAGE_INIT_NOT_PRESENT         (0x00010404) /* Initialization image not present */
#define IOP_LOGINFO_CODE_IMAGE_INIT_ADDRESS_ERROR       (0x00010405) /* Start address and image size of initialization image do not meet restrictions */
#define IOP_LOGINFO_CODE_IMAGE_NOT_MODEL_TO_FWDL        (0x00010406) /* Host tries to do firmware download when codes are run from flash */
#define IOP_LOGINFO_CODE_IMAGE_FLASH_SIZE_NOT_MATCH     (0x00010407) /* Flash size does not match total size in complete firmware download request */

/*istwi error */
#define IOP_LOGINFO_CODE_ISTWI_BUS_NOT_RESERVED         (0x00090000) /* ISTWI bus is not reserved */
#define IOP_LOGINFO_CODE_ISTWI_DEVICE_NOT_PRESENT       (0x00090001) /* ISTWI device not present */
#define IOP_LOGINFO_CODE_ISTWI_BUS_RESERVED_TWICE       (0x00090002) /* ISTWI bus reserved twice */
#define IOP_LOGINFO_CODE_ISTWI_INVALID_SGL              (0x00090003) /* Invalid SGL for ISTWI device */
#define IOP_LOGINFO_CODE_ISTWI_INVALID_SUBSYSTEM        (0x00090004) /* Invalid subsystem in ISTWI request */


#define IOP_LOGINFO_CODE_PWR_MGMT                       (0x000A0000) /* Any well-defined, known power management status */
#define IOP_LOGINFO_CODE_UNDEF_PWR_MGMT                 (0x000A8000) /* An undefined power management status */
#define IOP_LOGINFO_SUBCODE_LINK_SPEED_TRAINING_ERROR   (0x00000001) /* A request to change the PCIe link speed failed due to PCIe training error */
#define IOP_LOGINFO_SUBCODE_PCIE_LINK_ALREADY_HAD_ERROR (0x00000002) /* A request to change the PCIe link width failed, the link had a training error before we did anything to it */
#define IOP_LOGINFO_SUBCODE_LINK_WIDTH_TRAINING_ERROR   (0x00000003) /* We tried to change the PCIe link width, but got a link training error after attempting the change */
#define IOP_LOGINFO_SUBCODE_INVALID_PCIE_LINK_SPEED     (0x00000004) /* Bad PCIe link speed field in the request message */
#define IOP_LOGINFO_SUBCODE_INVALID_PCIE_LINK_WIDTH     (0x00000005) /* Bad PCIe link width field in the request message */
#define IOP_LOGINFO_SUBCODE_HW_GATEWAY_DISABLE_FAILED   (0x00000006) /* Unable to disable the AHB gateway */
#define IOP_LOGINFO_SUBCODE_RESULT_DIFFERS_FROM_REQUEST (0x00000007) /* The requested PCIe link speed/width != actual link speed/width */
#define IOP_LOGINFO_SUBCODE_HSM_NOT_SUPPORTED           (0x00000008) /* Half speed mode not supported because the CPU is not running @ 800MHz */
#define IOP_LOGINFO_SUBCODE_INVALID_CPU_SPEED           (0x00000009) /* The requested CPU speed is an invalid value */
#define IOP_LOGINFO_SUBCODE_INVALID_FEATURE             (0x0000000A) /* The value of the 'Feature' field in the MPI Power Management Control request is bad */

/****************************************************************************/
/* PL LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = PL            */
/****************************************************************************/

/* See pliIocLogInfo.h */


/****************************************************************************/
/* IR LOGINFO_CODE defines, valid if IOC_LOGINFO_ORIGINATOR = IR            */
/****************************************************************************/
#define IR_LOGINFO_RAID_ACTION_ERROR                           (0x00010000) /* RAID action error */
#define IR_LOGINFO_VOLUME_CREATE_INVALID_LENGTH                (0x00010001) /* Amount of information passed down to create volume is too large */
#define IR_LOGINFO_VOLUME_CREATE_NO_SLOTS                      (0x00010003) /* Creation failed due to exceeding the maximum number of supported volumes */
#define IR_LOGINFO_VOLUME_CREATE_DMA_ERROR                     (0x00010004) /* Creation failed due to DMA error when trying to read from host */
#define IR_LOGINFO_VOLUME_MFG_PAGE4_ERROR                      (0x00010006) /* Creation failed due to error reading manufacturing page 4 */
#define IR_LOGINFO_VOLUME_CREATE_NO_BGI                        (0x00010008) /* Creation failed due to not setting required Background Init flag */
#define IR_LOGINFO_VOLUME_CREATE_ODD_NUM_PHYS_DISKS            (0x00010009) /* Creation failed due to odd number drives in volume when not allowed */

#define IR_LOGINFO_VOLUME_ACTIVATING_AN_ACTIVE_VOLUME          (0x00010010) /* Activation failed due to trying to activate an already active volume */
#define IR_LOGINFO_VOLUME_ACTIVATING_INVALID_VOLUME_TYPE       (0x00010011) /* Activation failed due to trying to activate an unsupported volume type  */
#define IR_LOGINFO_VOLUME_ACTIVATING_TOO_MANY_VOLUMES          (0x00010012) /* Activation failed due to trying to activate too many volumes  */
#define IR_LOGINFO_VOLUME_ACTIVATE_VOLUME_FAILED               (0x00010014) /* Activation failed. Call to activateVolume returned failure */
#define IR_LOGINFO_VOLUME_ACTIVATING_IMPORT_VOLUME_FAILED      (0x00010015) /* Activation failed. Trying to import the volume */
#define IR_LOGINFO_VOLUME_ACTIVATING_TOO_MANY_PHYS_DISKS       (0x00010016) /* Activation failed. Trying to activate too many physical disks */
#define IR_LOGINFO_VOLUME_ACTIVATING_FAILED_STATE_VOLUME       (0x00010017) /* Activation failed. Volume is in failed state */
#define IR_LOGINFO_VOLUME_ACTIVATING_INVALID_METADATA_SIZE     (0x00010018) /* Activation failed. Activation of a volume with 64MB metadata is not allowed */
#define IR_LOGINFO_VOLUME_ACTIVATING_INVALID_PHYS_DISK         (0x00010019) /* Activation failed. Could not read physical disk pages */

#define IR_LOGINFO_PHYSDISK_CREATE_TOO_MANY_DISKS              (0x00010020) /* Creation physical disk failed, too many physical disks */
#define IR_LOGINFO_PHYSDISK_CREATE_CONFIG_PAGE_ERROR           (0x00010024) /* Creation failed due to error in creating RAID Physical Disk Configuration Page */
#define IR_LOGINFO_PHYSDISK_DUAL_PORT_FAILOVER                 (0x00010026) /* Physical disk IO returned: need time for remove device processing so the IO can be failed-over to passive path on this dual ported device */
#define IR_LOGINFO_PHYSDISK_CREATE_DEV_HANDLE_INVALID          (0x00010027) /* Creation failed due to invalid device handle */
#define IR_LOGINFO_PHYSDISK_CREATE_DUPLICATE_DEV_HANDLE        (0x00010028) /* Creation failed due to existing physical disk with this device handle */


#define IR_LOGINFO_COMPAT_ERROR_RAID_DISABLED                  (0x00010030) /* Compatibility Error : IR disabled */
#define IR_LOGINFO_COMPAT_ERROR_INQUIRY_FAILED                 (0x00010031) /* Compatibility Error : Inquiry Command failed */
#define IR_LOGINFO_COMPAT_ERROR_NOT_DIRECT_ACCESS              (0x00010032) /* Compatibility Error : Device is not direct access device */
#define IR_LOGINFO_COMPAT_ERROR_REMOVABLE_FOUND                (0x00010033) /* Compatibility Error : Removable device found */
#define IR_LOGINFO_COMPAT_ERROR_NEED_SCSI_2_OR_HIGHER          (0x00010034) /* Compatibility Error : Device SCSI Version not 2 or higher */
#define IR_LOGINFO_COMPAT_ERROR_SATA_48BIT_LBA_NOT_SUPPORTED   (0x00010035) /* Compatibility Error : SATA device, 48-bit LBA not supported */
#define IR_LOGINFO_COMPAT_ERROR_DEVICE_NOT_512_BYTE_BLOCK      (0x00010036) /* Compatibility Error : Device does not have 512-byte block sizes */
#define IR_LOGINFO_COMPAT_ERROR_VOLUME_TYPE_CHECK_FAILED       (0x00010037) /* Compatibility Error : Volume type check failed */
#define IR_LOGINFO_COMPAT_ERROR_DISK_TOO_SMALL                 (0x00010039) /* Compatibility Error : Disk drive too small for use in volume */
#define IR_LOGINFO_COMPAT_ERROR_MEMBERSHIP_COUNT               (0x0001003B) /* Compatibility Error : Membership count error, too many or too few disks for volume type */
#define IR_LOGINFO_COMPAT_ERROR_INCOMPATIBLE_STRIPE_SIZE       (0x0001003E) /* Compatibility Error : Disk stripe size must be 64KB or 128KB */
#define IR_LOGINFO_COMPAT_ERROR_UNSUPPORTED_PROTOCOL           (0x0001003F) /* Compatibility Error : Disk protocol type is unsupported by firmware */
#define IR_LOGINFO_COMPAT_ERROR_MIXED_PROTOCOL                 (0x00010040) /* Compatibility Error : Mixed protocol volumes are not supported */
#define IR_LOGINFO_COMPAT_ERROR_DISK_MAP                       (0x00010041) /* Compatibility Error : Physical disk map or RAID set number incorrect */
#define IR_LOGINFO_COMPAT_ERROR_DEVICE_NOT_PRESENT             (0x00010042) /* Compatibility Error : Device not present */
#define IR_LOGINFO_COMPAT_ERROR_NOT_END_DEVICE                 (0x00010043) /* Compatibility Error : Device not an end device */
#define IR_LOGINFO_COMPAT_ERROR_MEDIA_TYPE                     (0x00010044) /* Compatibility Error : Incompatible media */
#define IR_LOGINFO_COMPAT_ERROR_ALTA_NOT_SUPPORTED             (0x00010045) /* Compatibility Error : ALTA SAS-SATA Bridge detected */
#define IR_LOGINFO_COMPAT_ERROR_DRIVE_BLOCKED                  (0x00010046) /* Compatibility Error : >2.1 TB Drive detected and hence blocked */
#define IR_LOGINFO_COMPAT_ERROR_DRIVE_SENSE_DATA_INCOMPATIBLE  (0x00010047) /* Compatibility Error : Drive Sense Data format found incompatible and IR firmware was unable to change it. Drive did not allow IR firmware to change the Sense Data format */

#define IR_LOGINFO_DEV_FW_UPDATE_ERR_DFU_IN_PROGRESS           (0x00010050) /* Device Firmware Update: Can only be started once */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_DEVICE_IN_INVALID_STATE   (0x00010051) /* Device Firmware Update: Volume must be Optimal/Active/non-Quiesced */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_INVALID_TIMEOUT           (0x00010052) /* Device Firmware Update: Timeout value cannot be zero */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_NO_TIMERS                 (0x00010053) /* Device Firmware Update: Create timer failed */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_READING_CFG_PAGE          (0x00010054) /* Device Firmware Update: Failed to read SAS_IO_UNIT_Page_1 */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_PORT_IO_TIMEOUTS_REQUIRED (0x00010055) /* Device Firmware Update: Invalid SAS_IO_UNIT_Page_1 value(s) */
#define IR_LOGINFO_DEV_FW_UPDATE_ERR_ALLOC_CFG_PAGE            (0x00010056) /* Device Firmware Update: Unable to allocate memory for page */
/* Device Firmware Update:  */
//#define IR_LOGINFO_DEV_FW_UPDATE_ERR_                         (0x00010054)

#define IR_LOGINFO_DELETE_PHYSDISK_INVALID                      (0x00010070) /* Deletion failed due to invalid physical disk */
#define IR_LOGINFO_DELETE_HOT_SPARE_INVALID                     (0x00010071) /* Hot spare deletion failed due to invalid physical disk */

#define IR_LOGINFO_VOLUME_CREATE_FAILED_VOLUME_PRESENT          (0x00010075) /*Volume creation failed due to failed volume present*/
#define IR_LOGINFO_VOLUME_ACTIVATING_FAILED_VOLUME_PRESENT      (0x00010076) /*Volume activation failed due to failed volume present*/

#define IR_LOGINFO_RAID_ACTION_IR_DISABLED                      (0x00010080) /* RAID action failed due to IR disabled */

#if 0
/* Obsolete #defines - searched unused during cleanup */

//#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_PT         (0x00030400) /* Invalid Page Type */
//#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_DNM        (0x00030500) /* Device Not Mapped */
//#define IOP_LOGINFO_CODE_CONFIG_INVALID_PAGE_PERSIST    (0x00030600) /* Persistent Page not found */
//#define IR_LOGINFO_VOLUME_CREATE_DUPLICATE                     (0x00010002) /* Creation of duplicate volume attempted (Bus/Target ID checked) */
//#define IR_LOGINFO_VOLUME_CREATE_INVALID_VOLUME_TYPE           (0x00010005) /* Creation failed due to invalid volume type passed down */
//#define IR_LOGINFO_VOLUME_INTERNAL_CONFIG_STRUCTURE_ERROR      (0x00010007) /* Creation failed when trying to create internal structures */
//#define IR_LOGINFO_VOLUME_ACTIVATING_VOLUME_ID_IN_USE          (0x00010013) /* Activation failed due to Volume ID in use already */
//#define IR_LOGINFO_PHYSDISK_CREATE_INVALID_LENGTH              (0x00010021) /* Amount of information passed down for Create Pnysdisk is too large */
//#define IR_LOGINFO_PHYSDISK_CREATE_DMA_ERROR                   (0x00010022) /* Creation failed due to DMA error in trying to read from host */
//#define IR_LOGINFO_PHYSDISK_CREATE_BUS_TID_INVALID             (0x00010023) /* Creation failed due to invalid Bus TargetID passed down */
//#define IR_LOGINFO_PHYSDISK_CREATE_DUAL_PORT_ERROR             (0x00010025) /* Creation failed due to error: both ports of a Dual Ported device selected */
//#define IR_LOGINFO_COMPAT_ERROR_UNSUPPORTED_VOLUME_TYPE        (0x00010038) /* Compatibility Error : Volume Type is unsupported by firmware */
//#define IR_LOGINFO_COMPAT_ERROR_PHYS_DISK_NOT_FOUND            (0x0001003A) /* Compatibility Error : Phys disk for Create Volume not found */
//#define IR_LOGINFO_COMPAT_ERROR_NON_64K_STRIPE_SIZE            (0x0001003C) /* Compatibility Error : Disk stripe sizes must be 64KB */
//#define IR_LOGINFO_COMPAT_ERROR_IME_VOL_NOT_CURRENTLY_SUPPORTED (0x0001003D) /* Compatibility Error : IME size limited to < 2TB */

/* end Obsolete defines */
#endif

/****************************************************************************/
/* Defines for convenience                                                  */
/****************************************************************************/

/* See pliIocLogInfo.h */

#endif /* end of file */

