/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  iopiHw.h
 *          Title:  Header file with hardware related information
 *     Programmer:  Nilesh Wilanker
 *  Creation Date:  June 24, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  06/24/11  NW    Initial version.
 *  09/29/11  NW    CRAM space range is defined for IOC.
 *                  CRAM space range is modified for IOP, USB.
 *  01/28/12  JM    CRAM space range is defined for RAID.
 *  05/07/12  NB    Macro entries are made for all the modules uses CRAM.
 *  07/26/12  KK    CRAM space range is modified for IOC, RAID, Ethernet.
 *
 * Description
 * ------------
 * This file contains the CRAM hardware definitions.
 *
 *-------------------------------------------------------------------------
 */

#ifndef _IOPIHW_H_SOURCE
#define _IOPIHW_H_SOURCE

#define CRAM_START_ADDR             (0xC0000000)
#define CRAM_SIZE                   (0x400000)
#define CRAM_END_ADDR               (CRAM_START_ADDR + CRAM_SIZE)

#define IOP_CRAM_START_ADDR         (CRAM_START_ADDR)
#define IOP_CRAM_SIZE               (0x270000)
#define IOP_CRAM_END_ADDR           (IOP_CRAM_START_ADDR + IOP_CRAM_SIZE - 1)

#define IOC_CRAM_START_ADDR         (IOP_CRAM_END_ADDR + 1)
#define IOC_CRAM_SIZE               (0x95000)
#define IOC_CRAM_END_ADDR           (IOC_CRAM_START_ADDR + IOC_CRAM_SIZE - 1)

#define RAID_CRAM_START_ADDR        (IOC_CRAM_END_ADDR + 1)
#define RAID_CRAM_SIZE              (0x20000)
#define RAID_CRAM_END_ADDR          (RAID_CRAM_START_ADDR + RAID_CRAM_SIZE - 1)

#define USB_CRAM_START_ADDR         (RAID_CRAM_END_ADDR + 1)
#define USB_CRAM_SIZE               (0x2000)
#define USB_CRAM_END_ADDR           (USB_CRAM_START_ADDR + USB_CRAM_SIZE - 1)

#define ETH_CRAM_START_ADDR         (USB_CRAM_END_ADDR + 1)
#define ETH_CRAM_SIZE               (0xD9000)
#define ETH_CRAM_END_ADDR           (ETH_CRAM_START_ADDR + ETH_CRAM_SIZE - 1)

/* Add module entry which uses CRAM */
#define CRAM_IOP                    (0)
#define CRAM_IOC                    (1)
#define CRAM_ETH                    (2)
#define CRAM_RAID                   (3)
#define CRAM_USB                    (4)

#endif /* _IOPIHW_H_SOURCE */
