/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2012 LSI Corporation.  All rights reserved.              *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  i2cDefs.h
 *          Title:  I2C library header file.
 *     Programmer:  Jignesh Moradiya
 *  Creation Date:  April 27, 2012
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  04/27/12  JM    Initial version.
 *
 * Description
 * ------------
 * This file contains structure, definitions and function prototypes
 * for the I2C library.
 *
 *-------------------------------------------------------------------------
 */

#ifndef I2C_DEFS_H_SOURCE
#define I2C_DEFS_H_SOURCE

#include "basetype.h"

#define CONFIG_SYS_I2C_SLAVE    (0x50)
#define I2C_COUNT               (5)

/* TimeOut in Micro Seconds */
#define I2C_TIMEOUT_DEFAULT     (1000000)

#define I2C_FIFO_DEPTH          (0x08)

typedef enum _I2C_STATUS
{
    I2C_SUCCESS = 0,
    I2C_FAIL,
    I2C_NOK,                  /* I2C transfer failed */
    I2C_NOK_LA,               /* Lost arbitration    */
    I2C_NOK_ICT,              /* Incomplete transfer */
    I2C_NOK_XFRA,             /* Transfer aborted    */
    I2C_NOK_TOUT,             /* Transfer timeout    */
    I2C_IOREMAP_FAIL,         /* Ioremap failed      */
    I2C_INVALID_BUS,          /* Invalid bus number  */
    I2C_INVALID_SPD           /* Invalid speed       */
}I2C_STATUS;

typedef enum _I2C_SPEED
{
    I2C_100KHZ = 0,
    I2C_400KHZ
}I2C_SPEED;

typedef enum _I2C_BIT_ADDRESSING
{
    I2C_7_BIT_ADDR = 0,
    I2C_10_BIT_ADDR
}I2C_BIT_ADDRESSING;

typedef enum _I2C_BOOLEAN
{
    I2C_FALSE = 0,
    I2C_TRUE
}I2C_BOOLEAN;

typedef enum _I2C_BUS_STATUS
{
    I2C_LOW = 0,
    I2C_HIGH
}I2C_BUS_STATUS;

typedef enum _I2C_XFER_MODE
{
    I2C_MANUAL = 0,
    I2C_AUTO,
    I2C_SEQ,
    I2C_MONITOR
}I2C_XFER_MODE;

typedef struct _I2C_DEVICE
{
    U8 busNum;
    I2C_SPEED speed;
    U8 chip;
    I2C_BIT_ADDRESSING i2cBitAddr;
    I2C_XFER_MODE xferMode;
}I2C_DEVICE, *PTR_I2C_DEVICE;


I2C_STATUS i2ciSetTimeout ( U8 BusNum, U32 Timeout );
I2C_STATUS i2ciGetTimeout ( U8 BusNum, S32 *Timeout );
I2C_STATUS i2ciInit       ( void );
I2C_STATUS i2ciCleanup    ( void );
I2C_STATUS i2ciBusInit    ( U8 Busnum, I2C_SPEED Speed );
I2C_STATUS i2ciReset      ( U8 Busnum );
I2C_STATUS i2ciProbe      ( PTR_I2C_DEVICE );
I2C_STATUS i2ciRead       ( PTR_I2C_DEVICE, U32 Addr, U8 AddrLen, U8 *PtrBuffer, U16 *PtrLen );
I2C_STATUS i2ciWrite      ( PTR_I2C_DEVICE, U32 Addr, U8 AddrLen, U8 *PtrBuffer, U16 *PtrLen );
I2C_STATUS i2ciMasterStop ( U8 BusNum );
I2C_STATUS i2ciWriteSeq   ( PTR_I2C_DEVICE, U32 Addr, U8 AddrLen, U8* PtrRxData, U16 *PtrRxDataLen, U8* PtrTxData, U16 *PtrTxDataLen );

I2C_STATUS i2ciIsSDAFallEdgeDetected     ( U8 Busnum, I2C_BOOLEAN *PtrSdaFallEdgeSts );
I2C_STATUS i2ciIsSCLFallEdgeDetected     ( U8 Busnum, I2C_BOOLEAN *PtrSclFallEdgeSts );

I2C_STATUS i2ciClearSDAFallEdgeDetectSts ( U8 Busnum );
I2C_STATUS i2ciClearSCLFallEdgeDetectSts ( U8 Busnum );
I2C_STATUS i2ciSDAForceLow               ( U8 Busnum, U8 Val );
I2C_STATUS i2ciSCLForceLow               ( U8 Busnum, U8 Val );

I2C_STATUS i2ciSDAStatus                 ( U8 Busnum, I2C_BUS_STATUS *PtrSdaStatus );
I2C_STATUS i2ciSCLStatus                 ( U8 Busnum, I2C_BUS_STATUS *PtrSclStatus );

#endif /* I2C_DEFS_H_SOURCE*/
