/***************************************************************************
 *                                                                         *
 *  Copyright (C) 2011 LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  raidEngineDrv.h
 *          Title:  Raid Engine Driver Header File
 *     Programmer:  Naresh Bhalala
 *  Creation Date:  July 21, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   -------------------------------------------------------
 *  07/21/11  NKB   Initial OSS Compliance version.
 *  09/21/11  NKB   Structure for RAID Engine Driver is added.
 *  01/28/12  JM    osalUdelay() function is added.
 *
 *  Description
 *  ------------
 *  This file contains the definitions used for RAID Engine driver as
 *  an interface.
 *
 *-------------------------------------------------------------------------
 */

#ifndef RAID_ENGINE_DRV_H
#define RAID_ENGINE_DRV_H

#include <linux/module.h>

#include "basetype.h"

/* RAID Engine Driver - Status */
typedef enum _RE_STATUS
{
    RE_SUCCESS = 0,
    RE_FAILURE

} RE_STATUS;

/* Common Interface - RAID Accelerator */
typedef struct _RAID_ENGINE_DRIVER
{
    void *(*osalConvertPhysicalToVirtual) (unsigned long long);
    void (*osalMoveToDcr)                 (U32, U32);
    U32  (*osalMoveFromDcr)               (S32);
    void (*osalMemset)                    (void*, U8, U32);
    void *(*osalIoremap)                  (unsigned long long, U32);
    void (*osalIounmap)                   (void*);
    void (*osalUdelay)                    (U32);
    void (*iopiTracePrint0)               (long long Pattern, U32 Index);
    void (*iopiTracePrint1)               (long long Pattern, U32 Index, U32 Arg1);
    void (*iopiTracePrint2)               (long long Pattern, U32 Index, U32 Arg1, U32 Arg2);
    void (*iopiTracePrint3)               (long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3);
    void (*iopiTracePrint4)               (long long Pattern, U32 Index, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);
    void (*iopiConsolePrint0)             (char * FmtStr);
    void (*iopiConsolePrint1)             (char * FmtStr, U32 Arg1);
    void (*iopiConsolePrint2)             (char * FmtStr, U32 Arg1, U32 Arg2);
    void (*iopiConsolePrint3)             (char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3);
    void (*iopiConsolePrint4)             (char * FmtStr, U32 Arg1, U32 Arg2, U32 Arg3, U32 Arg4);

} RAID_ENGINE_DRIVER, *PTR_RAID_ENGINE_DRIVER;
extern RAID_ENGINE_DRIVER gRaidEngineDriver;

/* Exported Functions - Raid Engine Driver Functions */
extern RE_STATUS raidEngineRegisterDriver(ENGINE_DRIVER EngineDriver);
extern RE_STATUS raidEngineUnregisterDriver(U32 Major);

#endif /* RAID_ENGINE_DRV_H */
