/***************************************************************************
 *                                                                         *
 *  Copyright 2003 LSI Logic Corporation.  All rights reserved.            *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Logic.  The        *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  BASETYPE.H
 *          Title:  Header File for ARM base types
 *     Programmer:
 *  Creation Date:  October 23, 2002
 *
 *
 * Description
 * ------------
 *  This include file contains the ARM type definitions.  This file is
 *  designed for use with ARM966ES and ADS 2.5.
 *
 *-------------------------------------------------------------------------
 */

#ifndef BASETYPE_H_INCLUDED
#define BASETYPE_H_INCLUDED


#if  !defined(__LANGUAGE_ASM__)

#define EXTERNAL_MPI_TYPES_DEFINED 1
#if  EXTERNAL_MPI_TYPES_DEFINED
typedef signed   char       S8;
typedef unsigned char       U8;
typedef signed   short      S16;
typedef unsigned short      U16;
typedef signed   int        S32;
typedef unsigned int        U32;

typedef struct _S64
{
    U32                     Low;
    S32                     High;
} S64;

typedef struct _U64
{
    U32                     Low;
    U32                     High;
} U64;

/* Define Pointers to Basics Types  */

#endif /* EXTERNAL_MPI_TYPES_DEFINED */


#if  EXTERNAL_MPI_TYPES_DEFINED | CUSTOMER == 3601
/* Define Basic Types */
#define MPI2_POINTER        *

typedef struct _U96
{
    U32                     Dword0;
    U32                     Dword1;
    U32                     Dword2;
} U96;

typedef S8      *PS8;
typedef U8      *PU8;
typedef S16     *PS16;
typedef U16     *PU16;
typedef S32     *PS32;
typedef U32     *PU32;
typedef S64     *PS64;
typedef U64     *PU64;
typedef U96     *PU96;

#endif /* EXTERNAL_MPI_TYPES_DEFINED | CUSTOMER == 3601 */

typedef unsigned long long  U64math;

typedef struct _U128
{
    U32                     Dword[4];
} U128;
typedef U128    *PU128;

typedef struct _U256
{
    U32                     Dword[8];
} U256;
typedef U256    *PU256;


typedef U32     BOOL;


#endif /* !defined(__LANGUAGE_ASM__) */


/* Define some constant types */
#ifndef NULL
#define NULL    (0)
#endif

#ifndef TRUE
#define TRUE    (1)
#endif

#ifndef FALSE
#define FALSE   (0)
#endif

#define SUCCESS (1)
#define FAILURE (0)

#define ON      (TRUE)
#define OFF     (FALSE)

/* Define some macros */
#define MIN(a, b)   (((a) < (b)) ? (a) : (b))

#define MAX(a, b)   (((a) > (b)) ? (a) : (b))

#define DIFFERENCE(a, b) (((a) > (b)) ? ((a) - (b)) : ((b) - (a)))

/*  In order to determine the offset of an element of a structure, use the
 *  following macro as shown:
 *
 *  typedef struct _ABC
 *  {
 *      UINT    A;
 *      UINT    B;
 *      UINT    C;
 *  } ABC, * PTR_ABC
 *
 *  OFFSETOF( struct _ABC, C )
 */
#define  OFFSETOF( typ, id ) ((unsigned int)&(((typ*)0)->id))

/* Use this to prevent compiler warnings for unused parameters. The compiler
 * should be smart enough that this doesn't create any extra code.
 */
#define UNUSED_PARAMETER(Param) (Param=Param)

#ifdef __cplusplus
#define __clnk extern "C"
#else
#define __clnk
#endif

#if defined( __GNUC__ )
/* static definition required for GCC inline funcs to prevent multiple-definition errors across object modules */
#define INLINE static __inline__
#elif defined( __cplusplus )
#define INLINE static inline
#else
#define INLINE __inline
#endif

#if  !defined(__LANGUAGE_ASM__) && !defined(MEM_TEST_CFG)
/*
 * These macros speed the copying of structures by avoiding intrinsic memmove calls
 * These macros require the structure be of sizeof(U32)*N where N == 2 to 4
 */
INLINE void MEMMOVE64( U32 *ptrSrc, U32 *ptrDst)
{
    *ptrDst = *ptrSrc;
    *(ptrDst+1) = *(ptrSrc+1);
}
#endif

#endif /* BASETYPE_H_INCLUDED */


