/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp. The          *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  onfiiDefs.h
 *          Title:  ONFI Interface defines
 *     Programmer:  Thibash Rajamani
 *  Creation Date:  June 7, 2011
 *
 *
 * Description
 * -----------
 *   Contains the interface defines for the ONFI Library.
 *
 *-------------------------------------------------------------------------
 */

#ifndef ONFI_IDEFS_H_SOURCE
#define ONFI_IDEFS_H_SOURCE

/*
 * For ONFI libraries, the versioning scheme used is
 * 0xAABBCCDD, where
 * AA = Phase
 * BB = Customer / Branch (250 for pre GCA)
 * CC = point release #
 * DD = developer build field (needs to be left as 0 in check-in/release code)
 *
 * When a SAS3FW phase GCA happens, the version will need to be updated (from 1.250.3.0 to 2.0.0.0, for example)
 * and a new ONFI lib release has to be done.
 */
#ifndef ONFI_LIBRARY_VERSION
#define ONFI_LIBRARY_VERSION   (0x00FA0700)
#endif

/****************************************************************************
 *
 *   Return Status Values
 *
 ****************************************************************************/
#ifndef ISTATUS_SUCCESS
typedef U32 ISTATUS;
#define ISTATUS_SUCCESS                                 (0)
#endif

#ifndef ISTATUS_FAILURE
#define ISTATUS_FAILURE                                 (1)
#endif

/* Flash Controller Module Returns */
#define ISTATUS_ONFI_FC_MAP10_CMD_TIMEOUT               (2)
#define ISTATUS_ONFI_FC_MAP10_INVALID_CMD               (3)
#define ISTATUS_ONFI_FC_MAP01_CMD_TIMEOUT               (4)
#define ISTATUS_ONFI_FC_MAP01_INSUFFICIENT_BUFFER       (5)
#define ISTATUS_ONFI_FC_DATA_DMA_TIMEOUT                (6)
#define ISTATUS_ONFI_FC_COMMAND_DMA_TIMEOUT             (7)
#define ISTATUS_ONFI_FC_DMA_ENABLE_TIMEOUT              (8)
#define ISTATUS_ONFI_FC_DMA_DISABLE_TIMEOUT             (9)
#define ISTATUS_ONFI_FC_INVALID_TRANSFER_MODE           (10)
/* Memory Offload Module Returns */
#define ISTATUS_ONFI_MO_CMD_TIMEOUT                     (11)
#define ISTATUS_ONFI_MO_READ_DATA_FULL_TIMEOUT          (12)
#define ISTATUS_ONFI_MO_WRITE_DATA_EMPTY_TIMEOUT        (13)
/* ONFI Interface Function Returns */
#define ISTATUS_ONFII_INIT_INVALID_CHANNEL_INDEX        (14)
#define ISTATUS_ONFII_INIT_BANK_RESET_FAILED            (15)
#define ISTATUS_ONFII_DEV_INFO_INVALID_PARAMETER        (16)
#define ISTATUS_ONFII_BANK_RESET_FAILED                 (17)
#define ISTATUS_ONFII_DMA_INSUFFICIENT_BUFFER           (18)
#define ISTATUS_ONFII_L3_CACHE_FLUSH_TIMEOUT            (19)
#define ISTATUS_ONFII_BAD_BLOCK_INVALID_PARAMETER       (20)
#define ISTATUS_ONFII_BAD_BLOCK_TRANS_MODE_INVALID      (21)
#define ISTATUS_ONFII_BAD_BLOCK_MALLOC_FAILED           (22)
#define ISTATUS_ONFII_OFFLOAD_INVALID_PARAMETER         (23)
#define ISTATUS_ONFII_OFFLOAD_ENABLE_DMA_FAILED         (24)
#define ISTATUS_ONFII_OFFLOAD_CREATE_BB_LIST_FAIL       (25)
#define ISTATUS_ONFII_OFFLOAD_SAVING_BB_LIST_FAIL       (26)

#define ISTATUS_ONFII_RESTORE_INVALID_PARAMETER         (30)
#define ISTATUS_ONFII_RESTORE_INVALID_OPERATION_MODE    (31)
#define ISTATUS_ONFII_RESTORE_ENABLE_DMA_FAILED         (32)
#define ISTATUS_ONFII_RESTORE_DESC_INVALID              (33)
#define ISTATUS_ONFII_RESTORE_TIMEDOUT                  (34)
#define ISTATUS_ONFII_RESTORE_DIS_MISCOMPARE            (35)
#define ISTATUS_ONFII_RESTORE_DMA_FAILED                (36)
#define ISTATUS_ONFII_RESTORE_NOTHING_TO_RESTORE        (37)

#define ONFI_ENCRYPTION_KEY_SIZE                    (4)
#define L3_CACHE_CONTROLLER_OPERATION_FLUSH         (0)
#define L3_CACHE_CONTROLLER_OPERATION_INVALIDATE    (1)

/* ONFI Interface Modes */
typedef enum _ONFI_INTERFACE_MODE
{
    MODE_ASYNC  = 0x0,
    MODE_SYNC   = 0x1,
    MODE_TOGGLE = 0x2

} ONFI_INTERFACE_MODE;

typedef struct _ONFI_CONFIG
{
    U8   InterfaceMode;      /* ONFI_INTERFACE_MODE */
    U8   DmaEnable;          /* 1 - DMA Enable, 2 - DMA Disable */
    U8   EccEnable;          /* 1 - ECC Enable, 2 - ECC Disable */
    U8   OperationMode;      /* 1 - Poll Mode, 2 - Interrupt Mode */
    U8   EncryptionEnable;   /* 1 - Encryption Enable, 2 - Encryption Disable */
    U8   OffloadPauseEnable; /* 1 - OffloadPause Enable, 2 - OffloadPause Disable */
    U8   Reserved[2];
    PU32 PtrEccFifo;
    PU32 PtrWriteFifo;
    PU32 PtrReadFifo;
    U32  EncryptionKey[ONFI_ENCRYPTION_KEY_SIZE];

} ONFI_CONFIG, * PTR_ONFI_CONFIG;

#define ONFI_CONFIG_OPERATION_MODE_POLL         (1)
#define ONFI_CONFIG_OPERATION_MODE_INTERRUPT    (2)

#define ONFI_CONFIG_DMA_ENABLE                  (1)
#define ONFI_CONFIG_DMA_DISABLE                 (2)

#define ONFI_CONFIG_ECC_ENABLE                  (1)
#define ONFI_CONFIG_ECC_DISABLE                 (2)

#define ONFI_CONFIG_OFFLOAD_PAUSE_ENABLE        (1)
#define ONFI_CONFIG_OFFLOAD_PAUSE_DISABLE       (2)

#define ONFI_CONFIG_ENCRYPTION_ENABLE           (1)
#define ONFI_CONFIG_ENCRYPTION_DISABLE          (2)

typedef struct _ONFI_FC_DEVICE_INFO
{
    U8  OnfiDevice;         /* 1 - Device is ONFI complaint */
    U8  BanksPerCont;       /* Number of Banks per Controller */
    U8  LunsPerBank;        /* Number of Luns per Bank */
    U8  Reserved[1];
    U32 DeviceId;           /* Device ID */
    U32 BlocksPerLun;       /* Number of Blocks per Lun */
    U32 PagesPerBlock;      /* Number of Pages per Block */
    U32 BlockSize;          /* Block size */
    U32 DeviceWidth;        /* Device width */
    U32 SpareAreaSkipCount; /* ONFI spare area start offset */
    U32 SpareAreaSize;
    U32 MainAreaSize;
    U32 PageAddrBits;
    U32 PageAddrMask;
    U32 BlockAddrBits;
    U32 LunAddrBits;

} ONFI_FC_DEVICE_INFO, * PTR_ONFI_FC_DEVICE_INFO;

typedef struct _FC_IO_PARAM
{
    /* Generic */
    U8  Write;              /* 0 - Read, 1 - Write */
    U8  Reserved1;
    U8  ChannelIndex;
    U8  BankIndex;
    U32 BlockAddress;
    U32 PageAddress;
    U16 PageBufferOffset;
    U16 Reserved2;
    U32 TwoBeatDataMode;
    U32 FourBeatDataMode;

    /* Data DMA */
    U64 MemoryAddress;      /* Memory address to DMA */
    U16 PageCount;          /* Number of pages to DMA */
    U16 BurstLength;        /* Burst size */

    /* Command DMA */
    U64 DescAddress;        /* Descriptor address */
    U32 DMAIndex;

} FC_IO_PARAM, * PTR_FC_IO_PARAM;

#define ONFI_DATA_COPY_FLASH_TO_MEMORY  (0) /* Read operation */
#define ONFI_DATA_COPY_MEMORY_TO_FLASH  (1) /* Write operation */

typedef enum _ONFI_CHANNEL_INDEX
{
    ONFI_CHANNEL_0,
    ONFI_CHANNEL_1,
    ONFI_MAX_CHANNELS

} ONFI_CHANNEL_INDEX;

typedef enum _ONFI_BANK_INDEX
{
    ONFI_BANK_0,
    ONFI_BANK_1,
    ONFI_MAX_BANKS

} ONFI_BANK_INDEX;

/* Transfer Modes set in transfer_spare_reg register */
#define FC_TRANSFER_MODE_DEFAULT            0x00 /* Main only */
#define FC_TRANSFER_MODE_SPARE              0x01
#define FC_TRANSFER_MODE_MAIN_SPARE         0x02

/* 3 bit DMA Transfer Mode */
typedef enum _CMD_DMA_TRANSFER_MODE
{
    CMD_DMA_TRANSFER_MODE_DEFAULT       = 0, /* Gets mode value from transfer_spare_reg register */
    CMD_DMA_TRANSFER_MODE_MAIN_SPARE    = 1,
    CMD_DMA_TRANSFER_MODE_SPARE         = 2

} CMD_DMA_TRANSFER_MODE;

typedef struct _DMA_DESC_CMD_FLAG
{
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
    U16 Reserved            : 1;
    U16 TransferMode        : 3; /* CMD_DMA_TRANSFER_MODE */
    U16 WaitForMemcopy      : 1;
    U16 MemcopyAddrValid    : 1;
    U16 NextPointerValid    : 1;
    U16 GenerateIntOnComp   : 1;
    U16 BurstLength         : 8;
#else
    U16 BurstLength         : 8;
    U16 GenerateIntOnComp   : 1;
    U16 NextPointerValid    : 1;
    U16 MemcopyAddrValid    : 1;
    U16 WaitForMemcopy      : 1;
    U16 TransferMode        : 3; /* CMD_DMA_TRANSFER_MODE */
    U16 Reserved            : 1;
#endif
} DMA_DESC_CMD_FLAG, * PTR_DMA_DESC_CMD_FLAG;

typedef struct _DMA_DESC_SYNC_ARG
{
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
    U32 StartValue      : 8;
    U32 EndValue        : 8;
    U32 Reserved2       :11;
    U32 SyncInt         : 1;
    U32 Reserved1       : 1;
    U32 Valid           : 1;
    U32 Greater         : 1;
    U32 Increment       : 1;
#else
    U32  Increment      : 1;
    U32  Greater        : 1;
    U32  Valid          : 1;
    U32  Reserved1      : 1;
    U32  SyncInt        : 1;
    U32  Reserved2      :11;
    U32  EndValue       : 8;
    U32  StartValue     : 8;
#endif
} DMA_DESC_SYNC_ARG, * PTR_DMA_DESC_SYNC_ARG;

typedef struct _DMA_DESC_STATUS
{
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
    U16 Complete        : 1;
    U16 Fail            : 1;
    U16 Reserved2       : 5;
    U16 MaxError        : 5;
    U16 Reserved1       : 1;
    U16 LockedBlock     : 1;
    U16 UnsupportedCmd  : 1;
    U16 AccessError     : 1;
#else
    U16 AccessError     : 1;
    U16 UnsupportedCmd  : 1;
    U16 LockedBlock     : 1;
    U16 Reserved1       : 1;
    U16 MaxError        : 5;
    U16 Reserved2       : 5;
    U16 Fail            : 1;
    U16 Complete        : 1;
#endif
} DMA_DESC_STATUS, * PTR_DMA_DESC_STATUS;

typedef struct _DMA_DESC
{
    U64                 NextPointer;
    U32                 FlashPointer;
    U32                 Reserved1;
    U16                 CommandType;    /* MAP10_CMD_DATA */
    DMA_DESC_CMD_FLAG   CommandFlags;   /* This member is U16 and uses bit field, be sure about compiler alignment */
    DMA_DESC_SYNC_ARG   SyncArgs;
    U64                 MemoryAddress;
    DMA_DESC_STATUS     Status;         /* This member is U16 and uses bit field, be sure about compiler alignment */
    U16                 Reserved2;
    U32                 Reserved3;
    U64                 SyncFlagPointer;
    U64                 MemcopyPointer;
    U64                 Reserved4;

} DMA_DESC, * PTR_DMA_DESC;

/* Bit to identify Mem->Flash (1-write) or Flash->Mem (0-read)*/
#define ONFI_DMA_DESC_CMD_TYPE_WRITE_MASK     (0x100)
#define ONFI_DMA_DESC_NEXT_PTR_HIGH_ADDR_MASK (0x3)

extern ONFI_CONFIG gOnfiConfig[ONFI_MAX_CHANNELS];
extern ONFI_FC_DEVICE_INFO gOnfiDeviceInfo[ONFI_MAX_CHANNELS];

#endif
