/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp. The          *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  onfiiProtos.h
 *          Title:  ONFI Interface Function Prototypes
 *     Programmer:  Thibash Rajamani
 *  Creation Date:  June 7, 2011
 *
 *
 * Description
 * -----------
 *
 *
 *-------------------------------------------------------------------------
 */

#ifndef ONFI_IPROTOS_H_SOURCE
#define ONFI_IPROTOS_H_SOURCE

/* ONFI Flash Controller Interface Functions */
ISTATUS onfiiFlashContInit(U32 ChannelIndex, PTR_ONFI_CONFIG PtrOnfiConfig);
ISTATUS onfiiFlashContSetConfig(U32 ChannelIndex, PTR_ONFI_CONFIG PtrOnfiConfig);
ISTATUS onfiiFlashContDeviceInfo(U32 ChannelIndex, PTR_ONFI_FC_DEVICE_INFO PtrOnfiFcDeviceInfo);
ISTATUS onfiiFlashContBankReset(U32 ChannelIndex, U32 BankIndex);
void    onfiiFlashContReset(void);

ISTATUS onfiiFlashContBlockErase(U32 ChannelIndex, U32 BankIndex, U32 BlockAddress);
ISTATUS onfiiFlashContDwordReadWrite(PTR_FC_IO_PARAM PtrFcIoParam, PU64 PtrData);
ISTATUS onfiiFlashContPageReadWrite(PTR_FC_IO_PARAM PtrFcIoParam, PU32 PtrBuffer, U32 BufferSize);
U8      onfiiFlashContGetTransferMode(U32 ChannelIndex, U32 BankIndex);
ISTATUS onfiiFlashContSetTransferMode(U32 ChannelIndex, U32 BankIndex, U8 TransferMode);
U32     onfiiFlashContGetPageTransferSize(U32 ChannelIndex, U32 BankIndex);

ISTATUS onfiiFlashContDataDma(PTR_FC_IO_PARAM PtrFcIoParam, PU32 PtrDdrBuffer, U32 ddrBufferSize);
ISTATUS onfiiFlashContCommandDma(PTR_FC_IO_PARAM PtrFcIoParam);
void    onfiiFlashContPrepareCommandDmaDesc(PTR_FC_IO_PARAM PtrFcIoParam, PTR_DMA_DESC PtrDmaDescAddr,
                                            U64 PhysDataAddr);

void    onfiiDdrL3CacheFlushOrInvalidate(void * Address, U32 Size, U8 Operation);
ISTATUS onfiiDdrL3CacheFullFlush(void);

U32     onfiiMoveFromDCR(U32 DCRAddress);
void    onfiiMoveToDCR(U32 Value, U32 DCRAddress);

/* ONFI Memory Offload Interface Functions */
ISTATUS onfiiMemOffloadCreateBadBlockList(PU32 PtrBadBlockInfoBuffer, U32 SizeOfBuffer, U32 ChannelIndex);
BOOL    onfiiMemOffloadTestBadBlockBit(PU32 PtrBadBlockInfoBuffer, U32 ChannelIndex, U32 BankIndex, U32 BlockNum);
void    onfiiMemOffloadSetBadBlockBit(PU32 PtrBadBlockInfoBuffer, U32 ChannelIndex, U32 BankIndex, U32 BlockNum);
U32     onfiiMemOffloadGetBadBlockListMemReq(U32 ChannelIndex);

ISTATUS onfiiMemOffloadHandleError(U32 ChannelIndex);
void    onfiiMemOffloadResetDIS(U32 ChannelIndex, U32 BankIndex);
void    onfiiMemOffloadGetDIS(U32 ChannelIndex, U32 BankIndex, PU64 PtrDIS);

ISTATUS onfiiMemOffloadSetupHW(U32 ChannelIndex, U32 BankIndex, PTR_DMA_DESC PtrDmaDescList);
ISTATUS onfiiMemOffloadBank(U64 DdrStartAddr, U32 DdrSize, PTR_DMA_DESC PtrDescChain,
                            U32 BankNum, U32 ChannelNum, PU32 PtrBadBlockListBuf);
ISTATUS onfiiMemOffloadMirrored(U64 DdrStartAddr, U32 DdrSize, PTR_DMA_DESC PtrDescChain,
                                PU32 PtrBadBlockList, BOOL UpdateBadBlockList, U32 ChannelA,
                                U32 BankA,U32 ChannelB, U32 BankB);

/* ONFI Memory Restore Interface Functions */
ISTATUS onfiiMemRestore(PU32 PtrRestoreDescriptorListArr, U32 TimeoutUs);
ISTATUS onfiiMemRestoreBank(PTR_DMA_DESC PtrDescChain, U32 BankNum, U32 ChannelNum, U32 TimeoutSec);
ISTATUS onfiiMemRestoreMirrored(PTR_DMA_DESC PtrDescChain, U32 ChannelA, U32 BankA, U32 ChannelB, U32 BankB, U32 TimeoutSec);

#endif
