/***************************************************************************
 *                                                                         *
 *  Copyright 2011 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLibHal.h
 *          Title:  USB Library HAL header.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains definitions to be used by the USB HAL source code.
 *
 *-------------------------------------------------------------------------
 */

#ifndef _USBLIBHAL_H_
#define _USBLIBHAL_H_

#if (IOC_USB_SUPPORT)

#include "usbLibStatus.h"

#define USB_WAIT_MECHANISM_TIMER_BASED              0
//#define USB_WAIT_MECHANISM_POLLING_BASED            1

#define USB_ULPI_READ                               (0x00000000)
#define USB_ULPI_WRITE                              (0x20000000)
#define USB_ULPI_ADDR_SHIFT                         (16)
#define USB_ULPI_RD_SHIFT                           (8)
#define USB_ULPI_RD_MASK                            (0x0000ff00)
#define USB_ULPI_RW_RUN                             (0x40000000)

/* Definition of the Configurable Macros */

//#define USB_ASYNCHRONOUS_PARK_MODE              0 /* 1- enable 0- disable*/
//#define USB_AYSNCHRONOUS_PARK_MODE_COUNT        3
//#define USB_ENABLE_ASYNCHRONOUS_SCHEDULE        1
//#define USB_ENABLE_PERIODIC_SCHEDULE            0
//#define USB_FRAME_LIST_SIZE                     3
//#define USB_INTERRUPT_INTERVAL                  4 /* number of micro frames */
//#define USB_INTERRUPT_ON_ASYNC_SCHEDULE         1 /* 1 - enable 0 - disable */

//#define USB_PARALLEL_TRANSCEIVER                1
//#define USB_SERIAL_TRANSCEIVER                  0
//#define USB_PARALLEL_TRANSCEIVER_WIDTH          1

#define USB_WAKE_ON_OC                          1
#define USB_WAKE_ON_DISCONN                     1
#define USB_WAKE_ON_CONN                        1
#define USB_VBUS_POWER                          0
//#define USB_STREAM_DISABLE                      1

#define USB_ENDIAN_SELECT_LITTLE                0
#define USB_ENDIAN_SELECT_BIG                   1

//#define USB_CONTROLLER_HOST                     3

#define USB_ENABLE_PHY_CLOCK                    0
#define USB_PORT_FORCE_FULL_SPEED               0

/* Definitions of the USB Structures */

/* Que Head type definitions */
#define USB_HEAD_TYPE_ISO                       0
#define USB_HEAD_TYPE_QH                        1
#define USB_HEAD_TYPE_SPLIT                     2
#define USB_HEAD_TYPE_FSTN                      3

/* Port speed type */
#define USB_PORT_SPEED_FULL                       0
#define USB_PORT_SPEED_LOW                        1
#define USB_PORT_SPEED_HIGH                       2

/* PID type codes */
#define USB_PID_TYPE_OUT                        0
#define USB_PID_TYPE_IN                         1
#define USB_PID_TYPE_SETUP                      2


/* USB Run Control definitions */
#define USB_RUN_CONTROL_START                   1
#define USB_RUN_CONTROL_STOP                    0

/* USB Timer Type Single/Multi Shot */
#define USB_TIMER_TYPE_SINGLE_SHOT              0
#define USB_TIMER_TYPE_MULTI_SHOT               1

/* USB Async Shedule Enable/Diable flags */
#define USB_ASYNC_SCHEDULE_DISABLE              0
#define USB_ASYNC_SCHEDULE_ENABLE               1

/* USB Async Shedule Enable/Diable flags */
#define USB_INT_ON_ASYNC_ADVANCE_DISABLE        0
#define USB_INT_ON_ASYNC_ADVANCE_ENABLE         1

/* USB Port power states */
#define USB_PORT_POWER_ON                       1
#define USB_PORT_POWER_OFF                      0

/* USB status values */
#define USB_TRANSFER_ERORR_XACT_ERROR           0x08
#define USB_TRANSFER_ERROR_BABBLE               0x10
#define USB_TRANSFER_ERROR_DATA_BUFFER          0x20
#define USB_TRANSFER_ERROR_HALTED               0x40

/* Queue head index defines */
#define USB_QUEUE_HEAD_INDEX_CONTROL            0
#define USB_QUEUE_HEAD_INDEX_IN                 2
#define USB_QUEUE_HEAD_INDEX_OUT                1

#define USB_QUEUE_HEAD_MULT_ONE_PER_FRAME       1  /* 1 transaction per micro frame */

#define USB_QUEUE_ADDRESS_MASK                  0xfffffff8
#define USB_QUEUE_ADDRESS_SHIFT                 5

/* Queue Element Transfers Descriptor */
typedef struct _USB_DESC_QUEUE_TRANSFER_ELEMENT
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         NextQTDPtr : 27;
            U32         Reserved   :  4;
            U32         Terminate  :  1;
#else
            U32         Terminate  :  1;
            U32         Reserved   :  4;
            U32         NextQTDPtr : 27;
#endif
        } Fields;

        U32             DWord;

    } NxtLinkPtr;

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         AltNextQTDPtr : 27;
            U32         Reserved      :  4;
            U32         Terminate     :  1;
#else
            U32         Terminate     :  1;
            U32         Reserved      :  4;
            U32         AltNextQTDPtr : 27;
#endif
        } Fields;

        U32             DWord;

    } AltNxtLinkPtr;

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         DT              :  1;
            U32         TotalTransBytes : 15;
            U32         IOC             :  1;
            U32         C_Page          :  3;
            U32         CErr            :  2;
            U32         PIDCode         :  2;
            U32         Status          :  8;
#else
            U32         Status          :  8;
            U32         PIDCode         :  2;
            U32         CErr            :  2;
            U32         C_Page          :  3;
            U32         IOC             :  1;
            U32         TotalTransBytes : 15;
            U32         DT              :  1;
#endif
        } Fields;

        U32             DWord;

    } QTDToken;

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         BufferPtr     : 20;
            U32         CurrentOffset : 12;

#else
            U32         CurrentOffset : 12;
            U32         BufferPtr     : 20;
#endif
        } Fields;

        U32             DWord;

    } BufferPtr[5];

} USB_DESC_QUEUE_TRANSFER_ELEMENT, *PTR_USB_DESC_QUEUE_TRANSFER_ELEMENT;


/* USB descriptor of Queue Head */
typedef struct _USB_DESC_QUEUE_HEAD
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         NextQTDPtr : 27;
            U32         Reserved   :  2;
            U32         Type       :  2;
            U32         Terminate  :  1;
#else
            U32         Terminate  :  1;
            U32         Type       :  2;
            U32         Reserved   :  2;
            U32         NextQTDPtr : 27;
#endif
        } Fields;

        U32             DWord;

    } NxtQHPtr;  // Dword 0

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         NakReloadCnt       :  3;
            U32         CtrlEPFlag         :  1;
            U32         MaxPacketLength    : 12;
            U32         HeadOfReclList     :  1;
            U32         DTC                :  1;
            U32         EPSpeed            :  2;
            U32         EndPoint           :  4;
            U32         InactiveOnNxtTrans :  1;
            U32         DevAddress         :  7;
#else
            U32         DevAddress         :  7;
            U32         InactiveOnNxtTrans :  1;
            U32         EndPoint           :  4;
            U32         EPSpeed            :  2;
            U32         DTC                :  1;
            U32         HeadOfReclList     :  1;
            U32         MaxPacketLength    : 12;
            U32         CtrlEPFlag         :  1;
            U32         NakReloadCnt       :  3;
#endif
        } Fields;

        U32             DWord;

    } EPCap1;   // dword 1

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         Mult       :  2;
            U32         PortNum    :  7;
            U32         HubAddr    :  7;
            U32         FrameCMask :  8;
            U32         FrameSMask :  8;
#else
            U32         FrameSMask :  8;
            U32         FrameCMask :  8;
            U32         HubAddr    :  7;
            U32         PortNum    :  7;
            U32         Mult       :  2;
#endif
        } Fields;

        U32             DWord;

    } EPCap2;   // dword 2

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         CurrQTDPtr : 29;
            U32         Reserved   :  3;
#else
            U32         Reserved   :  3;
            U32         CurrQTDPtr : 29;
#endif
        } Fields;

        U32             DWord;

    } CurrentQTDPtr;     // dword 3

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         NextQTDPtr : 27;
            U32         Reserved   :  4; /*CHECK */
            U32         Terminate  :  1;
#else
            U32         Terminate  :  1;
            U32         Reserved   :  4; /*CHECK */
            U32         NextQTDPtr : 27;
#endif
        } Fields;

        U32             DWord;

    } NxtLinkPtr;   // dword 4

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         AltNextQTDPtr : 27;
            U32         Reserved      :  4;
            U32         Terminate     :  1;
#else
            U32         Terminate     :  1;
            U32         Reserved      :  4;
            U32         AltNextQTDPtr : 27;
#endif
        } Fields;

        U32             DWord;

    } AltNxtLinkPtr;    // dword 5

    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         DT              :  1;
            U32         TotalTransBytes : 15;
            U32         IOC             :  1;
            U32         C_Page          :  3;
            U32         CErr            :  2;
            U32         PIDCode         :  2;
            U32         Status          :  8;
#else
            U32         Status          :  8;
            U32         PIDCode         :  2;
            U32         CErr            :  2;
            U32         C_Page          :  3;
            U32         IOC             :  1;
            U32         TotalTransBytes : 15;
            U32         DT              :  1;
#endif
        } Fields;

        U32             DWord;

    } QTDToken;     // dword 6


    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32         BufferPtr     : 20;
            U32         CurrentOffset : 12;
#else
            U32         CurrentOffset : 12;
            U32         BufferPtr     : 20;
#endif
        } Fields;

        U32             DWord;

    } BufferPtr[5];     // dword 7 - B

//      USB_DESC_QUEUE_TRANSFER_ELEMENT TransferOverlay;
    U32                 Reserved[4];

} USB_DESC_QUEUE_HEAD, *PTR_USB_DESC_QUEUE_HEAD;

/* USB Port Status */
typedef struct _USB_PORT_STATUS
{
    U32                 TxSchHealth;

    U32                 PortSpeed;
    U32                 LineStatus;
    U32                 HighSpeedPort;
    U32                 OCChange;
    U32                 OCActive;
    U32                 PortEnChange;
    U32                 PortEnStatus;
    U32                 PortConnChange;
    U32                 PortConnStatus;

} USB_PORT_STATUS, *PTR_USB_PORT_STATUS; //USBStatus;

/* ************************************************************************** */
/* Function prototypes                                                        */
/* ************************************************************************** */

/* USB core top level functions */
void usbHalInitRegisters( U32 UsbPort );

USB_STATUS usbHalGetNumPorts( PU32 PtrUsbNumPorts );
USB_STATUS usbHalControllerReset( U32 UsbPort );
USB_STATUS usbHalInitUlpiPhy( U32 UsbPort );
USB_STATUS usbHalBusReset( U32 UsbPort );
USB_STATUS usbHalRunControl( U32 UsbPort, U32 StartStop );
USB_STATUS usbHalCoreSuspend( U32 UsbPort );
USB_STATUS usbHalCoreResume( U32 UsbPort  );

USB_STATUS usbHalDoRecoveryReset( U32 UsbPort );

/* USB port related functions */
USB_STATUS usbHalPortControl(
    U32                         UsbPort,
    U32                         PortNum,
    BOOL                        Enable
    );

USB_STATUS usbHalPortPowerControl(
    U32                         UsbPort,
    U32                         PortNum,
    BOOL                        PowerOn
    );

USB_STATUS usbHalPortStatusClear(
    U32                         UsbPort,
    PTR_USB_PORT_STATUS         PtrPortStatus
    );

USB_STATUS usbHalPortReset(
    U32                         UsbPort,
    U32                         PortNum
    );

USB_STATUS usbHalGetPortStatusReg(
    U32                         UsbPort,
    PTR_USB_PORT_REGISTER       PtrUsbPortReg
    );

/* USB timer functions */
USB_STATUS usbHalTimerReset( U32 UsbPort, PBAM_USB_GP_TIMER_NUMBER TimerNumber );
USB_STATUS usbHalTimerConfig(
    U32 UsbPort,
    PBAM_USB_GP_TIMER_NUMBER TimerNumber,
    U32                         TimerInMicroSec,
    U32                         TimerType
    );

USB_STATUS usbHalTimerStart( U32 UsbPort, PBAM_USB_GP_TIMER_NUMBER TimerNumber );
USB_STATUS usbHalTimerStop( U32 UsbPort, PBAM_USB_GP_TIMER_NUMBER TimerNumber );
USB_STATUS usbHalSingleShotTimer( U32 UsbPort, U32 TimeDelayInMsec );
USB_STATUS usbHalStartIoTimer( U32 UsbPort, U32 IoTimeoutMs );

/* USB interrupt related functions */
USB_STATUS usbHalGetStatusReg( U32 UsbPort, PTR_USB_STATUS_REG PtrUsbIntStatus );
USB_STATUS usbHalSetInterruptRegister( U32 UsbPort );
USB_STATUS usbHalClearStatusReg( U32 UsbPort );

/* USB connection related functions */
USB_STATUS usbHalSetPortSpeed( U32 UsbPort, U32 ConnectionSpeed );
USB_STATUS usbHalGetPortSpeed( U32 UsbPort, PU32 PtrConnectionSpeed );
USB_STATUS usbHalSetBurstLength( U32 UsbPort, U32 TxBurstLength, U32 RxBurstLength );

/* USB core low level register access functions */
void usbAsyncListAddrWrite( U32 UsbPort, U32 Address );
USB_STATUS usbHalAsyncScheduleEnableDisable( U32 UsbPort, BOOL EnableDisable );
void usbHalSetControllerMode( U32 UsbPort, U32 UsbControllerMode );


#endif /* #if (IOC_USB_SUPPORT) */

#endif /* #ifndef _USBLIBHAL_H_ */


