/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.	   *
 *  The receipt of or possession of this file does not convey any rights   *
 *  to reproduce or disclose its contents or to manufacture, use, or sell  *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  invader.h
 *          Title:  Common header file for Invader board
 *     Programmer:  Nilesh Wilanker
 *     Modified  :  Naresh Madhusudana
 *  Creation Date:  July 4, 2011
 *
 *  Version History
 *  ---------------
 *
 *  Date      Who   Description
 *  --------  ---   --------------------------------------------------------
 *  07/04/11  NW    Initial version.
 *  08/23/11  NW    Macro definitions required by iopMemset() to initialize 
                    DDR3.
 *  10/05/11  NM    Porting the file for LSISAS3108
 *  05/20/12  JM    Alignment changes, Thunderbolt -> LSISAS3108
 *
 * Description
 * ------------
 * File containing the common definitions for LSISAS3108 board
 *
 *--------------------------------------------------------------------------
 */

#ifndef _INVADER_H
#define _INVADER_H

/* Processor register names */
#define r0  0
#define r1  1
#define r2  2
#define r3  3
#define r4  4
#define r5  5
#define r6  6
#define r7  7
#define r8  8
#define r9  9
#define r10 10
#define r11 11
#define r12 12
#define r13 13
#define r14 14
#define r15 15
#define r16 16
#define r17 17
#define r18 18
#define r19 19
#define r20 20
#define r21 21
#define r22 22
#define r23 23
#define r24 24
#define r25 25
#define r26 26
#define r27 27
#define r28 28
#define r29 29
#define r30 30
#define r31 31

/* CCR Registers : System PLL Control (SysPllCtrl) DCR Address 0x300 */
#define dcr_ccr_pll_ctrl                (0x300)
#define dcr_ccr_mempllctrl              (0x301)
#define dcr_ccr_modcrst                 (0x30c)
#define dcr_ccr_modcrst1                (0x30d)
#define dcr_ccr_srsd                    (0x340)
#define MEM_DIV_VALUE                   (0x00000F00) /* 8-11 System PLL feedback divider */
#define PLB_DIV_VALUE                   (0x00700000) /* 20-22 System PLL PLB divider    */
#define MEM_PLL_BYPASS_VALUE            (0x00000080) /* 7 bit to check SysPllBypass */
#define HALF_SPEED_MODE                 (0x40000000) /* 30 bit to check HSM */
#define MODSPCRST_DMC_ALL               (0x00380000) /* Don't reset DMC test mux. */
#define MODSPCRST_TRINITY_ALL           (0x03000000) /* Reset Trinity */

#define CHIP_REVID_DCR_ADDR             (0x305)

/* Optimus Registers */
#define OPTIMUS_READY_REGISTER1         (0x1A9)
#define OPTIMUS_READY_REGISTER2         (0x1AA)
#define OPTIMUS_READY_REGISTER3         (0x1E3)

/* DDR3 Min Size */
#define DDR_MINIMUM_SIZE                (256)
#define SPD_SIZE                        (256)
#define DDR_SIZE_MB                     (0x100000)
#define DDR_PAGE_NUM_MAX                (128)

/* UART Registers */
#define LSR_THREMPTY                    (0x20)       /* fifo is empty (in fifo mode ) */
#define THR_REG                         (0x00)       /* Transmit holding reg */
#define LSR_REG                         (0x14)       /* Line status reg */
#define COM1_ADDRESS                    (0xC2100600) /* UART address */

#define CONFIG_SYS_SDRAM_BASE          (0x00000000)           /* _must_ be 0 */
#define CONFIG_DDR3_BASE0              CONFIG_SYS_SDRAM_BASE  /* DDR3 Base address is 0x0 */
#define CONFIG_DMA_VER_MEM_CONTROLLER  (0xc70f0000)
#define CONFIG_DMA_PHY_MEM_CONTROLLER  (0xc80f0000)
#define CONFIG_VIRT_MEM_CONTROLLER     (0xc70e0000)
#define CONFIG_PHY_MEM_CONTROLLER      (0xc80e0000)

#define MEM_STRUCTURE_ALIGN(Pointer,size)   ( (((U32)(Pointer)) + (size)-1) \
                                               & ~((size)-1) )
#define MC_CDB_EA_BASE                      (0xC00174e0)

#define CCR_DCR_MEMDIV		                (0xE001)

/* the 12 digit is enough to represent 32 bit integer */
#define PRINT_BUF_LEN                       (12)

#define stringify(s)    tostring(s)
#define tostring(s)     #s

#define mtspr(rn, v)    ({asm volatile("mtspr " stringify(rn) ",%0" : : "r" (v));})

#define mfspr(rn)       ({unsigned int rval; \
                        asm volatile("mfspr %0," stringify(rn) \
                        : "=r" (rval)); rval;})

#define mfdcr(rn)	    ({unsigned int rval;\
                        asm volatile("mfdcrx %0,%1" : "=r" (rval) : "r" (rn));rval;})

#define mtdcr(rn, v)	({asm volatile("mtdcrx %0, %1" : : "r" (rn), "r" (v));})


#define PPC476_TBLR_SPR         (0x10C)
#define PPC476_GET_TICKS()      mfspr(PPC476_TBLR_SPR)
#define PPC476_CHECK_TICKS(NextStamp) \
       ( ~(PPC476_GET_TICKS() - (NextStamp)) >> 31 )
#define TICKS_TO_USEC	1000
#endif /* _INVADER_H */
